/* downloads_page.c
*
*  populates download page in notepad and deals with its functions
*
*  (c) 2001-2002 Tim-Philipp Muller <t.i.m@orange.net>
*
*/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// TODO:
// - FIX downloads_column_resized ()


#include <time.h>

#include "global.h"
#include "guicoreconn.h"
#include "gaps.h"
#include "http_get.h"
#include "misc.h"
#include "misc_gtk.h"
#include "icons.h"
#include "mainwindow.h"
#include "blacklist.h"
#include "statusbar.h"
#include "status_page.h"
#include "gui_options.h"
#include "downloads_data.h"
#include "downloads_page.h"
#include "gui_flags.h"
#include "send_command.h"
#include "parse_message.h"


// ----- global variables

GtkWidget	*clist_downloads;
GtkWidget	*dl_scrollwin;		// global, so we can save adjustment on list redraw

float		 total_downloaded = 0;
time_t		 seconds_download_was_nonzero = 1;	// so we don't divide by zero


// ----- local variables and defines

// Two list of the downloads to cancel (in clist->selection style).
//	If we have multiple selection, we might have cases
//	where we need confirmation for _some_ downloads only,
//	but not for others, thus we have two lists.
static GList	*dl_cancellist_immediately = NULL;
static GList	*dl_cancellist_confirm = NULL;

static GtkWidget *downloads_page_stats_label = NULL;

enum
{
 DL_MENUITEM_SET_LOW_PRIO = 0,
 DL_MENUITEM_SET_NORMAL_PRIO,
 DL_MENUITEM_SET_HIGH_PRIO,
 DL_MENUITEM_PAUSE_SELECTED,
 DL_MENUITEM_RESUME_SELECTED,
 DL_MENUITEM_PAUSE_ALL,
 DL_MENUITEM_RESUME_ALL,
 DL_MENUITEM_CANCEL_SELECTED,
 DL_MENUITEM_HIDE_COLUMN,
 DL_MENUITEM_SHOW_ALL_COLUMNS,
 DL_MENUITEM_LINK_TO_LOG,
 DL_MENUITEM_SHOWHIDE_JUNK,
 DL_MENUITEM_TOGGLE_KEEPEXT,
 DL_MENUITEM_TOGGLE_COLROWTEXT,
 DL_MENUITEM_VERIFYCANCELS,
 DL_MENUITEM_TOGGLE_SAVE_CORRUPTED,
 DL_MENUITEM_TOGGLE_NO_ACTIVE_CANCEL,
 DL_MENUITEM_NO_IDS_PAUSE,
 DL_MENUITEM_TOGGLE_PROGRESSBAR,
 DL_MENUITEM_TOGGLE_VISIBLEONLY,
 DL_MENUITEM_TOGGLE_GREENPBPARTS,
 DL_MENUITEM_TOGGLE_BLACKLISTONDL,
 DL_MENUITEM_TOGGLE_DLTRANSIN100,
 DL_MENUITEM_CLEAR_SELECTION,
 DL_MENUITEM_SELECT_ALL,
 DL_MENUITEM_TOGGLE_UNSELECT_AFTER_ACTION,
 DL_MENUITEM_TOGGLE_SELMODE,
 DL_MENUITEM_TOGGLE_SHOW_PART_FN,
 DL_MENUITEM_PREVIEW_SELECTED,
 DL_MENUITEM_CLEAR_COMPLETED,
 DL_MENUITEM_JIGLE_SL_BROWSER,
 DL_MENUITEM_INVALID,
};

#define DL_MENUITEM_FUNC_PAUSE_ALL_IDLE_FLAG		(0x01<<16)
#define DL_MENUITEM_FUNC_PAUSE_ALL_WO_SOURCES_FLAG	(0x02<<16)
#define DL_MENUITEM_FUNC_PAUSE_ALL_NOT_FULLY_AVAIL_FLAG	(0x04<<16)
#define DL_MENUITEM_FUNC_BLACKLIST_FLAG			(0x08<<16)

#define DL_AVAIL_COL_NO		8	// so we only need to make one change if we change it

// ----- local functions declarations

static void		 downloads_cancel_really_cancel_callback (GtkWidget *widget, gpointer data);
static void		 downloads_cancel_donot_cancel_callback (GtkWidget *widget, gpointer data);
static void		 downloads_popupmenu_item_selected (GtkWidget *widget, gpointer data);
static GtkWidget	*downloads_popup_options_menu (guint row, guint col);
static void		 downloads_popup_menu_position_func (GtkMenu *menu, gint *x, gint *y, gpointer user_data);
static void		 downloads_popup_menu (guint row, guint col, guint button, guint32 activate_time);
static void		 downloads_column_resized (GtkCList *widget, int col, int new_width, gpointer data);
static gint		 downloads_key_pressed (GtkWidget *widget, GdkEvent *event);
static void		 downloads_scroll_vertical (GtkCList *clist, GtkScrollType scroll_type, gfloat pos, gpointer data);
static GtkWidget	*downloads_page_create_scrolled_clist (void);

// ----- functions

// downloads_cancel_really_cancel_callback
//
// This is called when the 'Yes, cancel them' button has been clicked
//	in the download cancel confirm dialog, or when we have downloads
//	to cancel immediately.
//
// Note: widget can be NULL. If it is not, it contains the dialog widget.

static void
downloads_cancel_really_cancel_callback (GtkWidget *widget, gpointer data)
{
	gboolean	 blacklist = FALSE;
	GList		*node = NULL;

	// "ok" button clicked. now destroy window
        if (widget) gtk_widget_destroy (GTK_WIDGET (data));

	// If dl_cancellist_immediately is set, we've been called from the cancel function
	//	to cancel some downloads immediately.
	if (dl_cancellist_immediately)
	{
		blacklist = (GPOINTER_TO_UINT(dl_cancellist_immediately->data)!=0);	// skip first element (=> the ->next)
		PROCESS_CLIST_SELECTION_LIST_CUSTOM_LIST(DL_CLIST, dl_cancellist_immediately->next, downloads, downloadRecord,
			if (item)
			{
				send_cancel_download (item->hash);
				if (blacklist) add_to_blacklist (item->hash, item->size, item->nojunkname);
				statusbar_extra_message (N_("%s%s."), _(" Download cancelled"), (blacklist) ? _(" and added to blacklist") : N_(""));
				item->cancelled = 0x01;
			}
		);
		if (dl_cancellist_immediately)
			g_list_free (dl_cancellist_immediately);
		dl_cancellist_immediately = NULL;
		update_space_stat = 1;
		return;
	}

	// If dl_cancellist_immediately is not set, we've been called by the
	//	confirmation dialog

	g_return_if_fail (dl_cancellist_confirm!=NULL);

	blacklist = (GPOINTER_TO_UINT(dl_cancellist_confirm->data)!=0);

	// Note: dl_cancellist_confirm does not contain row numbers but copies of hashes!
	//	(otherwise the removal of immediately to cancel downloads will re-arrange
	//	the clist and the downloadlist and neither row numbers nor pointers to
	//	filelist items would be valid anymore when the confirmation is actually
	//	given.
	node = dl_cancellist_confirm->next; // skip first element (=> the ->next)
	while (node)
	{
		guint8	*hash = (guint8*) node->data;
		if (hash)
		{
			downloadRecord *item = downloads_find_from_hash (hash);
			if (item)
			{
				send_cancel_download (item->hash);
				if (blacklist) add_to_blacklist (item->hash, item->size, item->nojunkname);
				statusbar_extra_message (N_("%s%s."), _(" Download cancelled"), (blacklist) ? _(" and added to blacklist") : N_(""));
				item->cancelled = 0x01;
			}
			g_free(hash);
		}
		node = node->next;
	}
	if (dl_cancellist_confirm) g_list_free (dl_cancellist_confirm);
	dl_cancellist_confirm = NULL;
	update_space_stat = 1;

}


// downloads_cancel_donot_cancel_callback
//
// Abort cancel from within cancel Yes/No dialog.
// data will contain the widget of the Yes/No dialog.

static void
downloads_cancel_donot_cancel_callback (GtkWidget *widget, gpointer data)
{
        if (data) gtk_widget_destroy (GTK_WIDGET (data));	// okay button clicked. now destroy dialog window
	// and do nothing, because we do NOT want to cancel..... => delete the cancellists
	if (dl_cancellist_immediately) g_list_free(dl_cancellist_immediately);
	if (dl_cancellist_confirm)  g_list_free(dl_cancellist_immediately);
	dl_cancellist_immediately = NULL;
	dl_cancellist_confirm = NULL;
}


// downloads_clist_column_clicked
//
//

static void
downloads_clist_column_clicked (GtkCList *clist, gint column, gpointer data)
{

	/* column that is already active for sorting? -> change sort type */

	if (column == downloadlist_sortcolumn)
	{
		if (downloadlist_sorttype == GTK_SORT_ASCENDING)
			downloadlist_sorttype = GTK_SORT_DESCENDING;
		else
			downloadlist_sorttype = GTK_SORT_ASCENDING;
	} else {
		/* set new sort column; default type = ascending */
		downloadlist_sortcolumn = column;
		downloadlist_sorttype   = GTK_SORT_ASCENDING;
	}

	downloadsclist_resort();
}


// downloads_pause_and_resume_all_active_downloads
//
//

void
downloads_pause_and_resume_all_active_downloads (void)
{
	GList	*active_downloads = NULL;
	static time_t	lastIPchange = 0;

	if (time(NULL)-lastIPchange >= 60*60)
	{
		lastIPchange = time(NULL);
		if (downloads)
		{
			PROCESS_GPTRARRAY (downloads, downloadRecord,
				if ((item) && item->status!=STATUS_PAUSED && item->status!=STATUS_PAUSING)
				{
					active_downloads = g_list_append (active_downloads, (gpointer)item);
					send_pause_download (item->hash);	// pause all active downloads
					send_disconnect();					// disconnect from server
					usleep(500);
					send_connect_to_server (0, 0);		// connect to any server
				}
			);
			if (active_downloads)
			{
				PROCESS_GLIST (active_downloads, downloadRecord,
					if (item)
						send_resume_download (item->hash);
				);
				g_list_free(active_downloads);
			}
			statusbar_extra_message (N_("%s"), _(" IP change signalled - paused and resumed all active downloads and changed server."));
		}
	} else g_printerr (N_("%s"), _(" You can't use |ipchanged| more often than once per hour.\n"));
}



// downloads_popupmenu_item_selected
//
// An item from the downloads pop-up menu has been selected. Take some action...
//
// data contains the menuitem selected (lowest 8 bits), and in some cases the column
//	where the click occured (second-lowest 8 bits), and some flags (second-highest 8 bits)
// Note: widget might be NULL

static void
downloads_popupmenu_item_selected (GtkWidget *widget, gpointer data)
{
	guint		menuitem =  GPOINTER_TO_UINT (data) & 0xff;
	guint		column   = (GPOINTER_TO_UINT (data) & 0xff00) >> 8;
	guint		flags    =  GPOINTER_TO_UINT (data) & 0xff0000;
	gboolean	pauseall_idle = ((flags&DL_MENUITEM_FUNC_PAUSE_ALL_IDLE_FLAG)!=0);
	gboolean	pauseall_nosrc = ((flags&DL_MENUITEM_FUNC_PAUSE_ALL_WO_SOURCES_FLAG)!=0);
	gboolean	pauseall_notfull = ((flags&DL_MENUITEM_FUNC_PAUSE_ALL_NOT_FULLY_AVAIL_FLAG)!=0);
	gboolean	blacklist = ((flags&DL_MENUITEM_FUNC_BLACKLIST_FLAG)!=0);
	gboolean	unselect_all = TRUE;
	guint		i;

	switch (menuitem)
	{
		case DL_MENUITEM_HIDE_COLUMN:
		{
			dlcols_visibility[column] = 0;
			gtk_clist_set_column_visibility (DL_CLIST, column, FALSE);
		}
		break;
		case DL_MENUITEM_SHOW_ALL_COLUMNS:
		{
			for (i=0; i<NO_OF_DOWNLOAD_COLS-1; i++)		// don't unhide last column (hash)!
			{
				dlcols_visibility[i] = 1;
				gtk_clist_set_column_visibility (DL_CLIST, i, TRUE);
			}
		}
		break;
		case DL_MENUITEM_SHOWHIDE_JUNK:
		{
			option_nofnjunk = (~option_nofnjunk) & 1;		// XOR 1
			downloadsclist_update_all_filenames ();
		}
		break;
		case DL_MENUITEM_SET_LOW_PRIO:
		{
			PROCESS_CLIST_SELECTION_LIST (DL_CLIST, downloads, downloadRecord,
				if (item)
				{
					send_set_download_priority (item->hash, 0);
					item->priority = 0;
					statusbar_extra_message (N_("%s"), _(" Set download priority to low."));
				}
			);
		}
		break;
		case DL_MENUITEM_SET_NORMAL_PRIO:
		{
			PROCESS_CLIST_SELECTION_LIST (DL_CLIST, downloads,  downloadRecord,
				if (item)
				{
					send_set_download_priority (item->hash, 1);
					item->priority = 1;
					statusbar_extra_message (N_("%s"), _(" Set download priority to normal."));
				}
			);
		}
		break;
		case DL_MENUITEM_SET_HIGH_PRIO:
		{
			PROCESS_CLIST_SELECTION_LIST (DL_CLIST, downloads, downloadRecord,
				if (item)
				{
					send_set_download_priority (item->hash, 2);
					item->priority = 2;
					statusbar_extra_message (N_("%s"), _(" Set download priority to high."));
				}
			);
		}
		break;

		case DL_MENUITEM_PREVIEW_SELECTED:
		{
			PROCESS_CLIST_SELECTION_LIST (DL_CLIST, downloads, downloadRecord,
				if (item) 
				{
					if (item->partnum!=0)
					{
						gchar *preview_filename = g_strdup_printf (N_("%s/%u.part"), option_temporary, item->partnum);
						if (preview_filename)
						{
							if (spawn_file_preview(preview_filename, item->format)!=0)
							{
								g_printerr (_("Could not preview '%s'!\n"), preview_filename);
							}
							g_free(preview_filename);
						}
					} else g_printerr (_("This core version does not supply us with the x.part filenames.\n"));
				}
			);
			PROCESS_CLIST_SELECTION_LIST (DL_CLIST, completed, downloadRecord,
				if (item)
				{
					gchar *preview_filename;
					preview_filename = NULL;
					if (item->status == STATUS_COMPLETE)
						preview_filename = g_strdup_printf (N_("%s/%s"), option_incoming, item->name);
					if (item->status == STATUS_CORRUPTED)
					{
						if (item->partnum>0)
						{
							preview_filename = g_strdup_printf (N_("%s/%u.part"), option_incoming, item->partnum);
						} else g_printerr (_("This core version does not supply us with the x.part filenames (needed to preview corrupted download).\n"));
					} 

					if (preview_filename)
					{
						if (spawn_file_preview(preview_filename, item->format)!=0)
						{
							g_printerr (_("Could not preview '%s'!\n"), preview_filename);
						}
						g_free(preview_filename);
					}
				}
			);
		}
		break;


		case DL_MENUITEM_PAUSE_SELECTED:
		{
			PROCESS_CLIST_SELECTION_LIST (DL_CLIST, downloads, downloadRecord,
				if ((item) && item->status!=STATUS_PAUSED && item->status!=STATUS_PAUSING)
				{
					send_pause_download (item->hash);
					item->status = STATUS_PAUSING;
					gtk_clist_set_text (DL_CLIST, row, 1, _("Pausing..."));	// 1 = second column
					update_space_stat = 1;
				}
			);
		}
		break;
		case DL_MENUITEM_RESUME_SELECTED:
		{
			PROCESS_CLIST_SELECTION_LIST (DL_CLIST, downloads, downloadRecord,
				if ((item) && (item->status==STATUS_PAUSED||item->status==STATUS_PAUSING))
				{
					send_resume_download (item->hash);
					item->status = STATUS_RESUMING;
					gtk_clist_set_text (DL_CLIST, row, 1, _("Resuming..."));	// 1 = second column
					update_space_stat = 1;
				}
			);
		}
		break;
		case DL_MENUITEM_PAUSE_ALL:
		{
			if (!downloads) break;
			for (i=0; i<downloads->len; i++)
			{
				downloadRecord *item = g_ptr_array_index (downloads, i);
				if (item->status!=STATUS_PAUSED && item->status!=STATUS_PAUSING)
				{
					guint	row = gtk_clist_find_row_from_data (DL_CLIST, (gpointer) item);
					// This looks all a bit messy, but it saves us from re-inventing the wheel
					// donotpause is TRUE if we don't want to pause this insuff. disk space download
					gboolean donotpause = ((option_noinsuffdspause) && item->status==STATUS_IDS);
					if ((pauseall_idle || pauseall_nosrc || pauseall_notfull) && (donotpause)) continue;
					if ((pauseall_idle) && item->speed > 0.0) continue;
					if ((pauseall_nosrc) &&	item->sources>0) continue;
					if ((pauseall_notfull) && item->availability >= 100) continue;
					send_pause_download (item->hash);
					gtk_clist_set_text (DL_CLIST, row, 1, _("Pausing..."));	// 1 = second column
					item->status = STATUS_PAUSING;
				}
			}
			update_space_stat = 1;
			if (pauseall_idle || pauseall_nosrc || pauseall_notfull)
			{
				statusbar_extra_message (N_("%s"), _(" All downloads paused that fitted the selected condition."));
			} else statusbar_extra_message (N_("%s"), _(" All downloads paused."));
		}
		break;
		case DL_MENUITEM_RESUME_ALL:
		{
			if (!downloads) break;
			for (i=0; i<downloads->len; i++)
			{
				downloadRecord *item = g_ptr_array_index (downloads, i);
				if (item->status==STATUS_PAUSED || item->status==STATUS_PAUSING)
				{
					guint row = gtk_clist_find_row_from_data (DL_CLIST, (gpointer) item);
					send_resume_download (item->hash);
					gtk_clist_set_text (DL_CLIST, row, 1, _("Resuming..."));	// 1 = second column
					item->status = STATUS_RESUMING;
				}
			}
			update_space_stat = 1;
			statusbar_extra_message (N_("%s"), _(" All downloads resumed."));
		}
		break;
		case DL_MENUITEM_CANCEL_SELECTED:
		{
			gboolean	 cancelerrdls = FALSE;	// trying to cancel downloads with error?
			gchar		*names = g_strdup(N_("\n"));

			// Note: we could have cases where only some cancels need to be confirmed, but not others
			//	(eg. with multiple selection and 'never cancel active downloads w/o confirmation' option)
			//	Therefore we have two cancellists.
			// First item in cancellists is flag whether the items should be blacklisted or not
			dl_cancellist_immediately = g_list_append (NULL, GUINT_TO_POINTER(blacklist));
			dl_cancellist_confirm = g_list_append (NULL, GUINT_TO_POINTER(blacklist));

			PROCESS_CLIST_SELECTION_LIST (DL_CLIST, downloads, downloadRecord,
				// If item is NULL, we'll get the dialog wrongly. But this case shouldn't happen anyway.
				if ((item) && item->status!=STATUS_ERRLOADING && item->status!=STATUS_ERRHASHING)
				{
					if (option_verify_cancels==1 || (option_noactivedlcancel==1 && item->status!=STATUS_PAUSED && item->status!=STATUS_PAUSING))
					{
						if ((item->nojunkname) || (item->name))
						{
							gchar *tmpstr = g_strconcat (names, (item->name) ? item->name : item->nojunkname, N_("\n"), NULL);
							if (names) g_free(names);
							names = tmpstr;
						}
						// Note: items->data is from PROCESS_CLIST_SELECTION_LIST macro
						// dl_cancellist_confirm needs to contain file HASHES, because
						//	rows and/or filelist_item pointers would be wrong
						//	if the 'immediately to cancel' downloads get removed before
						//	the confirmation for the others is given
						//	(which would usually be the case) !!!
						dl_cancellist_confirm = g_list_append (dl_cancellist_confirm, (gpointer) g_memdup(item->hash,16));
					} else dl_cancellist_immediately = g_list_append (dl_cancellist_immediately, items->data);
				} else cancelerrdls = TRUE;
			);
			if (cancelerrdls)
			{
				ok_dialog (_("Cancel download"), _("One or more downloads selected have either had\nan error during loading or\n"
				"an error during hashing.\n\nYou can't just cancel these downloads.\n\n"
				"You will have to remove them manually\n(when the donkey is not running).\n\n"
				"The right-click pop-up menu\nshould show you which x.part file\na certain download is.\n"));
			}

			// show confirm dialog for those downloads for which we need confirmation to cancel them
			if (g_list_length(dl_cancellist_confirm)>1)
			{
				gchar *msg = g_strdup_printf (N_("%s\n%s\n%s\n\n"), _("Are you sure you want to cancel the following downloads?"),
								names, _("The unfinished files will be deleted from your harddrive."));
				yesno_dialog (_("cancel download"), msg, _("   Yes, do it   "), _("   No, don't do it   "),
				 		downloads_cancel_really_cancel_callback,
				 		downloads_cancel_donot_cancel_callback);
				g_free(msg);
				g_free(names);
			} else {
				g_list_free(dl_cancellist_confirm);
				dl_cancellist_confirm = NULL;
			}

			// cancel all downloads for which we do not need confirmation right away...
			if (g_list_length(dl_cancellist_immediately)<=1)
			{
				g_list_free (dl_cancellist_immediately);
				dl_cancellist_immediately = NULL;
			} else downloads_cancel_really_cancel_callback (NULL, NULL);
		}
		break;

		case DL_MENUITEM_LINK_TO_LOG:
		{
			PROCESS_CLIST_SELECTION_LIST (DL_CLIST, downloads, downloadRecord,
				if ((item) && (item->name) && (item->hash))
				{
					status_msg_print_ed2klink (item->name, item->size, item->hash);
				}
			);
		}
		break;

		case DL_MENUITEM_VERIFYCANCELS:
			option_verify_cancels = (~option_verify_cancels) & 1;	// XOR 1
		break;

		case DL_MENUITEM_TOGGLE_KEEPEXT:
		{
			option_keep_extensions = (~option_keep_extensions) & 1;
			downloadsclist_update_all_filenames ();
		}
		break;

		case DL_MENUITEM_TOGGLE_SAVE_CORRUPTED:
			option_save_corrupted = (~option_save_corrupted) & 1;
			send_save_corrupted (option_save_corrupted);
		break;

		case DL_MENUITEM_NO_IDS_PAUSE:
			option_noinsuffdspause = (~option_noinsuffdspause) & 1;
		break;

		case DL_MENUITEM_TOGGLE_COLROWTEXT:
			option_color_whole_line_in_downloads = (~option_color_whole_line_in_downloads) & 1;
		break;

		case DL_MENUITEM_TOGGLE_NO_ACTIVE_CANCEL:
			option_noactivedlcancel = (~option_noactivedlcancel) & 1;
		break;

		case DL_MENUITEM_TOGGLE_PROGRESSBAR:
			option_progressbar = (~option_progressbar) & 1;
		break;

		case DL_MENUITEM_TOGGLE_VISIBLEONLY:
			option_visibleonly = (~option_visibleonly) & 1;
		break;

		case DL_MENUITEM_TOGGLE_GREENPBPARTS:
			option_greenshading = (~option_greenshading) & 1;
		break;

		case DL_MENUITEM_TOGGLE_BLACKLISTONDL:
			option_blacklistondl = (~option_blacklistondl) & 1;
		break;

		case DL_MENUITEM_TOGGLE_DLTRANSIN100:
			option_dltransinpercent = (~option_dltransinpercent) & 1;
		break;

		case DL_MENUITEM_CLEAR_SELECTION:
			gtk_clist_unselect_all (DL_CLIST);
		break;

		case DL_MENUITEM_SELECT_ALL:
			gtk_clist_select_all (DL_CLIST);
			unselect_all = FALSE;
		break;

		case DL_MENUITEM_TOGGLE_SELMODE:
		{
			option_DLSingleSelectionMode = (~option_DLSingleSelectionMode) & 1;
			// set multi- or single- selection mode
			if (option_DLSingleSelectionMode==1)
			{
				gtk_clist_set_selection_mode(DL_CLIST, GTK_SELECTION_SINGLE);
			} else gtk_clist_set_selection_mode(DL_CLIST, GTK_SELECTION_EXTENDED);
		}
		break;

		case DL_MENUITEM_TOGGLE_UNSELECT_AFTER_ACTION:
			option_DLUnselectAfterAction = (~option_DLUnselectAfterAction) & 1;
		break;

		case DL_MENUITEM_TOGGLE_SHOW_PART_FN:
		{
			option_DLShowPartFileName = (~option_DLShowPartFileName) & 1;
			downloadsclist_update_all_filenames ();
		}
		break;

		case DL_MENUITEM_CLEAR_COMPLETED:
		{
			downloads_clear_all_completed();
		}
		break;

		case DL_MENUITEM_JIGLE_SL_BROWSER:
		{
			PROCESS_CLIST_SELECTION_LIST (DL_CLIST, downloads, downloadRecord,
				if (item)
				{
					gchar *url = g_strdup_printf (N_("http://jigle.com/search?p=ed2k:%u:%s&ma=1&d=1&a=0&l=10&t=7&x=&sl=1&su=&v=1"), item->size, hash_to_hash_str(item->hash));
					if (url)
					{
						invoke_browser_with_url (url);
						http_get_retrieve_serverlist(url,0);
						g_free(url);
					}
				}
			);
		}
		break;

		default:
			g_printerr (N_("GUI: Fix me - called with unknown value %u (%s)\n)"), menuitem, __FUNCTION__);
		break;
	}

	if (option_DLUnselectAfterAction==1 && option_DLSingleSelectionMode==0 && (unselect_all))
	{
		gtk_clist_unselect_all (DL_CLIST);
	}
}

// downloads_popup_menu_position_func
//
// We want to position the pop-up menu a bit higher than the
// 	default function does - otherwise the full options submenu
// 	might not fit fully on the screen...

static void
downloads_popup_menu_position_func (GtkMenu *menu, gint *x, gint *y, gpointer user_data)
{
	GtkRequisition	requisition;
	gint		xpos, ypos, screen_width, screen_height;

	g_return_if_fail (menu != NULL);
	g_return_if_fail (GTK_IS_MENU (menu));

	gdk_window_get_pointer (NULL, &xpos, &ypos, NULL);
	gtk_widget_size_request (GTK_WIDGET (menu), &requisition);	// see GTK source code

	screen_width = gdk_screen_width ();
	screen_height = gdk_screen_height ();

	*x = CLAMP (xpos - 2, 0, MAX (0, screen_width - requisition.width));
	*y = CLAMP (ypos - 50, 0, MAX (0, screen_height - requisition.height));
}




// downloads_popup_options_menu
//
// Creates the options submenu for the pop-up menu for the download list
// row col:  clist row and column where click took place

static GtkWidget *
downloads_popup_options_menu (guint row, guint col)
{
	GtkWidget *menu = NULL;

	const gchar	*KEEP_EXT_STR =			_(" keep file extensions when removing junk in download list ");
	const gchar	*JUNK_STR =				_(" hide junk in filenames shown ");
	const gchar	*PART_FN_STR =			_(" show x.part filenames in list ");
	const gchar	*HIDE_COL_STR =			_(" hide this column (%s) ");
	const gchar	*UNHIDE_COLS_STR =		_(" unhide all columns ");
	const gchar	*CANCEL_VERIFY_STR =	_(" verify download cancels ");
	const gchar	*SAVE_CORRUPTED_STR = 	_(" keep/save corrupted downloads instead of deleting them (core) ");
	const gchar	*NO_ACTIVE_CANCEL_STR = _(" never cancel non-paused downloads without confirmation ");
	const gchar	*COL_ROW_TEXT_STR =		_(" colour download row text as well, not just the 'dots' ");
	const gchar	*NO_IDS_PAUSE_STR =		_(" skip files with Insuff.Disk Space on conditional 'pause all' ");
	const gchar	*SHOW_PROGRESS_BARS =	_(" show progress bars in download availability column ");
	const gchar	*UPDATE_ONLY_VISI_STR = _(" progress bars: update only currently visible rows (uses less CPU) ");
	const gchar	*GREEN_PBAR_PARTS_STR =	_(" progress bars: use green shading for available parts ");
	const gchar	*BLACKLIST_ON_DL_STR =	_(" add new downloads automatically to blacklist ");
	const gchar	*DL_TRANS_PERCENT_STR =	_(" show 'transfered' amount in download list as percent ");
	const gchar	*DL_USEL_AFTR_ACT_STR =	_(" unselect all items after each action ");
	const gchar	*DL_SINGLE_MODE_STR = 	_(" use single selection mode in download list ");

	gchar		**icon_yes = icon_optionon;
	gchar		**icon_no  = icon_optionoff;
	

	menu  = gtk_menu_new();

	// This assignment is not 'by the (reference) book', but we don't
	//	have any other choice to change the pos.func w/o popping
	//	up the menu, do we?

	(void)	MENU_ITEM_ADD (JUNK_STR,	downloads_popupmenu_item_selected, DL_MENUITEM_SHOWHIDE_JUNK, (option_nofnjunk==1) ? icon_yes : icon_no );
	(void)	MENU_ITEM_ADD (KEEP_EXT_STR,	downloads_popupmenu_item_selected, DL_MENUITEM_TOGGLE_KEEPEXT, (option_keep_extensions==1) ? icon_yes : icon_no);
	(void)	MENU_ITEM_ADD (PART_FN_STR,	downloads_popupmenu_item_selected, DL_MENUITEM_TOGGLE_SHOW_PART_FN, (option_DLShowPartFileName==1) ? icon_yes : icon_no);
			MENU_ITEM_ADD__________;
	if (col < NO_OF_DOWNLOAD_COLS)
	{
		const gchar	*colname = gtk_clist_get_column_title (DL_CLIST, col);
		gchar		*itemstr;
		if (!colname) colname = N_("???");
		itemstr = g_strdup_printf (HIDE_COL_STR, (*colname==' ') ? colname+1 : colname);
		(void) MENU_ITEM_ADD (itemstr, downloads_popupmenu_item_selected, DL_MENUITEM_HIDE_COLUMN | (col<<8), icon_columnhide);
		g_free (itemstr);
	}
	(void)	MENU_ITEM_ADD (UNHIDE_COLS_STR, downloads_popupmenu_item_selected, DL_MENUITEM_SHOW_ALL_COLUMNS, icon_columnshow);
			MENU_ITEM_ADD__________;
	(void)	MENU_ITEM_ADD (CANCEL_VERIFY_STR, downloads_popupmenu_item_selected, DL_MENUITEM_VERIFYCANCELS, (option_verify_cancels==1) ? icon_yes : icon_no);
	(void)	MENU_ITEM_ADD (NO_ACTIVE_CANCEL_STR, downloads_popupmenu_item_selected, DL_MENUITEM_TOGGLE_NO_ACTIVE_CANCEL, (option_noactivedlcancel==1) ? icon_yes : icon_no);
	(void)	MENU_ITEM_ADD (SAVE_CORRUPTED_STR, downloads_popupmenu_item_selected, DL_MENUITEM_TOGGLE_SAVE_CORRUPTED, (option_save_corrupted==1) ? icon_yes : icon_no);
	(void)	MENU_ITEM_ADD (NO_IDS_PAUSE_STR, downloads_popupmenu_item_selected, DL_MENUITEM_NO_IDS_PAUSE, (option_noinsuffdspause==1) ? icon_yes : icon_no);
			MENU_ITEM_ADD__________;
	(void)	MENU_ITEM_ADD (COL_ROW_TEXT_STR, downloads_popupmenu_item_selected, DL_MENUITEM_TOGGLE_COLROWTEXT, (option_color_whole_line_in_downloads==1) ? icon_yes : icon_no);
	(void)	MENU_ITEM_ADD (SHOW_PROGRESS_BARS, downloads_popupmenu_item_selected, DL_MENUITEM_TOGGLE_PROGRESSBAR, (option_progressbar==1) ? icon_yes : icon_no);
	(void)	MENU_ITEM_ADD (UPDATE_ONLY_VISI_STR, downloads_popupmenu_item_selected, DL_MENUITEM_TOGGLE_VISIBLEONLY, (option_visibleonly==1) ? icon_yes : icon_no);
	(void)	MENU_ITEM_ADD (GREEN_PBAR_PARTS_STR, downloads_popupmenu_item_selected, DL_MENUITEM_TOGGLE_GREENPBPARTS, (option_greenshading==1) ? icon_yes : icon_no);
	(void)	MENU_ITEM_ADD (BLACKLIST_ON_DL_STR, downloads_popupmenu_item_selected, DL_MENUITEM_TOGGLE_BLACKLISTONDL, (option_blacklistondl==1) ? icon_yes : icon_no);
	(void)	MENU_ITEM_ADD (DL_TRANS_PERCENT_STR, downloads_popupmenu_item_selected, DL_MENUITEM_TOGGLE_DLTRANSIN100, (option_dltransinpercent==1) ? icon_yes : icon_no);
			MENU_ITEM_ADD__________;
	(void)	MENU_ITEM_ADD (DL_USEL_AFTR_ACT_STR, downloads_popupmenu_item_selected, DL_MENUITEM_TOGGLE_UNSELECT_AFTER_ACTION, (option_DLUnselectAfterAction==1) ? icon_yes : icon_no);
	(void)	MENU_ITEM_ADD (DL_SINGLE_MODE_STR, downloads_popupmenu_item_selected, DL_MENUITEM_TOGGLE_SELMODE, (option_DLSingleSelectionMode==1) ? icon_yes : icon_no);
	return menu;
}


/* downloads_popup_priority_menu
 *
 * pops up a submenu for the priority stuff
 *
 */
static GtkWidget *
downloads_popup_priority_menu (guint row, guint col)
{
	const gchar	*PRIO_HIGH_STR =	_(" high ");
	const gchar	*PRIO_NORMAL_STR =	_(" normal ");
	const gchar	*PRIO_LOW_STR =		_(" low ");
	GtkWidget	*menu = NULL;

	menu  = gtk_menu_new();
	(void) MENU_ITEM_ADD (PRIO_HIGH_STR,	downloads_popupmenu_item_selected, DL_MENUITEM_SET_HIGH_PRIO, icon_threestars);
	(void) MENU_ITEM_ADD (PRIO_NORMAL_STR,	downloads_popupmenu_item_selected, DL_MENUITEM_SET_NORMAL_PRIO, icon_twostars);
	(void) MENU_ITEM_ADD (PRIO_LOW_STR,		downloads_popupmenu_item_selected, DL_MENUITEM_SET_LOW_PRIO, icon_onestar);

	return menu;
}


/* downloads_popup_pauseall_menu
 *
 * pops up a submenu for the pauseall stuff
 *
 */
static GtkWidget *
downloads_popup_pauseall_menu (guint row, guint col)
{
	const gchar	*PAUSE_IDLE_STR =	_(" pause all idle downloads ");
	const gchar	*PAUSE_NOSRCS_STR =	_(" pause all downloads w/o sources ");
	const gchar	*PAUSE_NOTFA_STR =	_(" pause all not fully available downloads ");
	const gchar	*PAUSE_ALL_STR =	_(" pause all downloads ");
	GtkWidget	*menu = NULL;

	menu  = gtk_menu_new();
	(void)	  MENU_ITEM_ADD (PAUSE_IDLE_STR,	downloads_popupmenu_item_selected, DL_MENUITEM_PAUSE_ALL|DL_MENUITEM_FUNC_PAUSE_ALL_IDLE_FLAG, icon_pauseall);
	(void)	  MENU_ITEM_ADD (PAUSE_NOSRCS_STR,downloads_popupmenu_item_selected, DL_MENUITEM_PAUSE_ALL|DL_MENUITEM_FUNC_PAUSE_ALL_WO_SOURCES_FLAG, icon_pauseall);
	(void)	  MENU_ITEM_ADD (PAUSE_NOTFA_STR,	downloads_popupmenu_item_selected, DL_MENUITEM_PAUSE_ALL|DL_MENUITEM_FUNC_PAUSE_ALL_NOT_FULLY_AVAIL_FLAG, icon_pauseall);
	(void)	  MENU_ITEM_ADD (PAUSE_ALL_STR,	downloads_popupmenu_item_selected, DL_MENUITEM_PAUSE_ALL, icon_pauseall);

	return menu;
}


/* downloads_popup_cancel_menu
 *
 * pops up a submenu for the cancel stuff
 *
 */
static GtkWidget *
downloads_popup_cancel_menu (guint row, guint col)
{
	const gchar	*CANCEL_STR =		_(" cancel selected downloads ");
	const gchar	*CANCEL_BLIST_STR =	_(" cancel and add to blacklist ");
	GtkWidget	*menu = NULL;

	menu = gtk_menu_new();
	(void) MENU_ITEM_ADD (CANCEL_STR,	downloads_popupmenu_item_selected, DL_MENUITEM_CANCEL_SELECTED , icon_cancel);
	(void) MENU_ITEM_ADD (CANCEL_BLIST_STR,downloads_popupmenu_item_selected, DL_MENUITEM_CANCEL_SELECTED|DL_MENUITEM_FUNC_BLACKLIST_FLAG , icon_skull);

	return menu;
}

/* downloads_popup_menu
 *
 * Creates a pop-up menu for the downloads list and pops it up
 * row, col: clist row and column where click took place
 *
 * CHANGED BY AXEL
 * depending on option_gnomeicons being set or not, we draw original icons or GNOME2 icons
 */

static void
downloads_popup_menu (guint row, guint col, guint button, guint32 activate_time)
{
	static GtkWidget	*menu = NULL;
	static GtkWidget	*mi[32];	/* we should be punished if we have more than 32 items in the pop-up menu */
	static GtkWidget	*optitem;
	static GtkWidget	*pauseallitem;

/*	const gchar	*SELECT_ALL_STR =	_(" select all ");
	const gchar	*CLEAR_SELECT_STR =	_(" unselect all ");
*/
	const gchar	*PRIO_STR =			_(" set priority ... ");
	const gchar	*PAUSE_STR =		_(" pause ");
	const gchar	*RESUME_STR =		_(" resume  ");
	const gchar	*CANCEL_STR =		_(" cancel ... ");
	const gchar	*RESUME_ALL_STR =	_(" resume all ");
	const gchar	*PAUSE_ALL_STR =	_(" pause all ... ");
	const gchar *PREVIEW_STR =		_(" preview ");
	const gchar *JIGLE_SL_STR =		_(" import and show jigle serverlist  ");
	const gchar	*LOG_LINK_STR =		_(" write ed2k-link to log/clipboard ");
	const gchar *CLEAR_COMPLETED =	_(" clear completed downloads ");
	const gchar	*DL_OPTIONS_STR = 	_(" downloads options ");
	gint		 n_clearcompleted, n_preview, n_prio, n_cancel, n; /* n_pauseall,  */

//	if (menu) gtk_widget_destroy(menu);

	menu  = gtk_menu_new();

	n = 0;
/*
	if (option_DLSingleSelectionMode==0)
	{
		(void)	MENU_ITEM_ADD (SELECT_ALL_STR,	downloads_popupmenu_item_selected, DL_MENUITEM_SELECT_ALL, icon_none);
	}
	mi[n++]	= MENU_ITEM_ADD (CLEAR_SELECT_STR,downloads_popupmenu_item_selected, DL_MENUITEM_CLEAR_SELECTION, icon_none);
	(void)	  MENU_ITEM_ADD__________;
*/

	/* pause */
	mi[n++]	= MENU_ITEM_ADD (PAUSE_STR,	downloads_popupmenu_item_selected, DL_MENUITEM_PAUSE_SELECTED, icon_pause);

	/* resume */
	mi[n++]	= MENU_ITEM_ADD (RESUME_STR,	downloads_popupmenu_item_selected, DL_MENUITEM_RESUME_SELECTED, icon_resume);

	/* preview */
	n_preview = n;
	mi[n++]	= MENU_ITEM_ADD (PREVIEW_STR,	downloads_popupmenu_item_selected, DL_MENUITEM_PREVIEW_SELECTED, icon_eyes);
	if (!CORE_IS_LOCAL)
		gtk_widget_set_sensitive (GTK_WIDGET(mi[n_preview]), FALSE);

	/* set priority ... */
	n_prio = n;
	mi[n++] = MENU_ITEM_ADD (PRIO_STR, NULL, 0, icon_twostars);
	gtk_menu_item_set_submenu (GTK_MENU_ITEM(mi[n_prio]), downloads_popup_priority_menu(row,col));

	/* cancel ... */
	n_cancel = n;
	mi[n++] = MENU_ITEM_ADD (CANCEL_STR, NULL, 0, icon_cancel);
	gtk_menu_item_set_submenu (GTK_MENU_ITEM(mi[n_cancel]), downloads_popup_cancel_menu(row,col));

	/* copy link to clipboard */
	mi[n++]	= MENU_ITEM_ADD (LOG_LINK_STR,	downloads_popupmenu_item_selected, DL_MENUITEM_LINK_TO_LOG, icon_status);
	(void)	  MENU_ITEM_ADD__________;

	/* pause all */
	pauseallitem = MENU_ITEM_ADD (PAUSE_ALL_STR, NULL, 0, icon_pauseall);
	gtk_menu_item_set_submenu (GTK_MENU_ITEM(pauseallitem), downloads_popup_pauseall_menu(row,col));

	/* resume all */
	(void)	  MENU_ITEM_ADD (RESUME_ALL_STR,	downloads_popupmenu_item_selected, DL_MENUITEM_RESUME_ALL, icon_resumeall);
	(void)	  MENU_ITEM_ADD__________;

	/* clear completed */
	n_clearcompleted = n;
	mi[n++]	= MENU_ITEM_ADD (CLEAR_COMPLETED,	downloads_popupmenu_item_selected, DL_MENUITEM_CLEAR_COMPLETED, icon_none);
	if ((!completed) || ((completed)&&(completed->len==0)))
		gtk_widget_set_sensitive(GTK_WIDGET(mi[n_clearcompleted]), FALSE);
	(void)	  MENU_ITEM_ADD__________;

	/* show jigle list */
	mi[n++]	= MENU_ITEM_ADD (JIGLE_SL_STR,downloads_popupmenu_item_selected, DL_MENUITEM_JIGLE_SL_BROWSER, icon_none);

	/* options */
	optitem = MENU_ITEM_ADD (DL_OPTIONS_STR, NULL, 0, icon_options);
	gtk_menu_item_set_submenu (GTK_MENU_ITEM(optitem), downloads_popup_options_menu(row,col));

	/* show x.part filename of the download selected, but only if
	 * - exactly one download is selected, and
	 * - the one download selected is not a 'completed' item
	 */
	if (g_list_length(GTK_CLIST(clist_downloads)->selection)==1)
	{
		guint			 item_row = GPOINTER_TO_UINT((GTK_CLIST(clist_downloads)->selection)->data);
		downloadRecord	*dl = (downloadRecord*) gtk_clist_get_row_data (GTK_CLIST(clist_downloads), item_row);
		if ((dl) && dl->status!=STATUS_COMPLETE && dl->status!=STATUS_CORRUPTED)
		{
			gchar		 partname_str[32];
			gchar		 left_str[128];		/* needs enough space for possible translations as well! */
			gchar		 info_str[32+128];
			GtkWidget	*info_menu_item;

			g_snprintf (partname_str, sizeof(partname_str), N_(" %u.part"), dl->partnum);
			if (dl->transferred <= dl->size)
			{
				g_snprintf(left_str, sizeof(left_str), _(" %s left"), (gchar*)human_size(dl->size-dl->transferred));
			} else left_str[0] = 0x00;
			g_snprintf (info_str, sizeof(info_str), N_("%s -%s"), partname_str, left_str);
			(void)			  MENU_ITEM_ADD__________;
			info_menu_item	= MENU_ITEM_ADD (info_str, NULL, 0, icon_none);
			gtk_widget_set_sensitive (info_menu_item, FALSE);
		}
	}

	// No items selected? => make those functions that need items selected insensitive
	// (but not the 'clear completed' item, because completed items can't be selected!)
	if (!DL_CLIST->selection)
	{
		guint i;
		for (i=0; i<n; i++)
		{
			if (i != n_clearcompleted)
				gtk_widget_set_sensitive (GTK_WIDGET(mi[i]), FALSE);
		}
	}

	gtk_menu_popup (GTK_MENU (menu), NULL, NULL, downloads_popup_menu_position_func, NULL, button, activate_time);
}


// downloads_button_pressed
//
// Shows the downloads pop-up menu on right-click on the clist
// Pauses/resumes download on double-click on row (if double-click not disabled in options)
//
// This right-click pop-up menu code was inspired by code posted by Mike Hicks to
//	the gtk-list@redhat.com mailing list on 25/11/98. Thank you, Mike :)

static gint
downloads_button_pressed (GtkWidget *widget, GdkEventButton *event)
{
	guint			row, col;

	last_time_of_user_click = time(NULL);

	if (event->type == GDK_BUTTON_PRESS)
	{
		// convert GdkEvent to GdkEventButton
//		GdkEventButton *bevent = (GdkEventButton *) event;
		if (event->button == 3)	// right-click
		{
			gtk_clist_get_selection_info (DL_CLIST, (int) event->x ,(int) event->y, &row, &col);
			// if nothing selected (or only one row), select row of click
			if ((DL_CLIST) && g_list_length(DL_CLIST->selection)<=1)
			{
				gtk_clist_unselect_all (DL_CLIST);
				gtk_clist_select_row (DL_CLIST, row, col);
			}
			downloads_popup_menu (row, col, event->button, event->time);

		}
		return TRUE;	 // event has been dealt with
	}

	// double-click? => toggle pause/resume download
	if ((event->type) == GDK_2BUTTON_PRESS  && option_disable_doubleclick==0)
	{
		gint num_completed = (completed!=NULL) ? completed->len : 0;
//		// convert GdkEvent to GdkEventButton
//		GdkEventButton *bevent = (GdkEventButton *) event;
		g_return_val_if_fail(DL_CLIST!=NULL,FALSE);
		gtk_clist_get_selection_info (DL_CLIST, (gint) event->x ,(gint) event->y, &row, &col);

		if ((downloads) && row < (downloads->len + num_completed) && row>=0)	// otherwise click on empty space
		{
			GList		*original_selection, *fake_selection;
			downloadRecord	*rowitem = (downloadRecord*) gtk_clist_get_row_data (DL_CLIST, row);
			g_return_val_if_fail (rowitem!=NULL,FALSE);

			original_selection = DL_CLIST->selection;
			// Make fake selection with only one item for downloads_popupmenu_item_selected()
			fake_selection = g_list_append (NULL, GUINT_TO_POINTER(row));
			DL_CLIST->selection = fake_selection;
			downloads_popupmenu_item_selected (NULL, GUINT_TO_POINTER((rowitem->status == STATUS_PAUSED) ? DL_MENUITEM_RESUME_SELECTED : DL_MENUITEM_PAUSE_SELECTED	));
			if (fake_selection) g_list_free (fake_selection);
			DL_CLIST->selection = original_selection;
		}
		return TRUE;
	}

	return FALSE;
}


// downloadclist_compare_func
//
// Compare two rows so that the clist can be sorted according to our wishes
// (we're using stuff from the gtk source gtkclist.c default_compare function here)

gint
downloadclist_compare_func (GtkCList *clist, gconstpointer ptr1, gconstpointer ptr2)
{
	downloadRecord	*dl1, *dl2;

	g_return_val_if_fail (ptr1!=NULL,0);				// just making sure...
	g_return_val_if_fail (ptr2!=NULL,0);

	dl1 = (downloadRecord*) ((GtkCListRow*)ptr1)->data;
	dl2 = (downloadRecord*) ((GtkCListRow*)ptr2)->data;

	g_return_val_if_fail (dl1!=NULL,0);					// just making sure...
	g_return_val_if_fail (dl2!=NULL,0);

	// sort depending on type of content (i.e. which column is the one to sort)
	switch (downloadlist_sortcolumn)
	{
		default:
		case 0:																// 0 - filename
		{
			if (option_nofnjunk==1)
			{
				MISC_RETURN_STR_COMPARE(dl1->nojunkname,dl2->nojunkname);
			} else MISC_RETURN_STR_COMPARE(dl1->name,dl2->name);
		}
		case 1:																// 1 - status
		{
			/* values are bogus, only the order matters */
			const gint vals[STATUS_PAUSING+1] = {40,4,2,1,6,96,97,-1,40,98,0,-1,50,99,1,2,6};	
			gint status1 = MIN(dl1->status,STATUS_PAUSING);
			gint status2 = MIN(dl2->status,STATUS_PAUSING);
			if (dl1->speed > 0.0) status1=STATUS_DOWNLOADING;
			if (dl2->speed > 0.0) status2=STATUS_DOWNLOADING;
			MISC_RETURN_NUM_COMPARE(vals[status1],vals[status2]);
		}

		case 2:	MISC_RETURN_NUM_COMPARE(dl1->size,dl2->size);				// 2 - size
		case 3:
		{
			if ((option_dltransinpercent) && (dl1->size>0) && (dl2->size>0))
			{
				MISC_RETURN_NUM_COMPARE((dl1->transferred*1.0)/(dl1->size*1.0), (dl2->transferred*1.0)/(dl2->size*1.0));
			} else MISC_RETURN_NUM_COMPARE(dl1->transferred,dl2->transferred);	// 3 - transferred
		}
		case 4:																// 4 - bytes left
		{
			MISC_RETURN_NUM_COMPARE((gint32)(dl1->size-dl1->transferred),(gint32)(dl2->size-dl2->transferred));
		}
		case 5:	MISC_RETURN_NUM_COMPARE(dl1->speed,dl2->speed);				// 5 - speed
		case 6: MISC_RETURN_NUM_COMPARE(dl1->secs_left,dl2->secs_left);		// 6 - time left
		case 7: MISC_RETURN_NUM_COMPARE(dl1->priority,dl2->priority);		// 7 - priority
		case DL_AVAIL_COL_NO:												// 8 - availability
		{
			MISC_RETURN_NUM_COMPARE(dl1->availability,dl2->availability);
		}
       	case 9:	MISC_RETURN_NUM_COMPARE(dl1->sources,dl2->sources);			// 9 - sources


		case 10: MISC_RETURN_NUM_COMPARE(dl1->lasttime,dl2->lasttime);		// 10 - last downloading (higher=more recent)
		case 11: 															// 11 - last complete higher=more recent)
		{
			MISC_RETURN_NUM_COMPARE(dl1->lasttime_complete,dl2->lasttime_complete);
		}
		case 12:															// 12 - hash
		{
			MISC_RETURN_NUM_COMPARE(GUINT32_TO_BE(*((guint32*)dl1->hash)),GUINT32_TO_BE(*((guint32*)dl2->hash)));
		}
	}
	g_assert_not_reached();	// never reached
}



// downloads_column_resized
//
// A column of the download list has been resized. Remember new value
//	so we can save it in the options file

static void
downloads_column_resized (GtkCList *widget, gint col, gint new_width, gpointer data)
{
	column_widths[DL_COLS][col] = new_width;
	if (col==DL_AVAIL_COL_NO && (downloads) && downloads->len>0)	// 2nd+3rd to avoid call on clist creation
	{
		guint i;
		// if avail. col resize, free all pixmaps, so they are redrawn
		for (i=0; i<downloads->len; i++)
			free_pixmaps (g_ptr_array_index(downloads,i));
	}
}


// downloads_key_pressed
//
// A key has been pressed in the downloads list.

static gint
downloads_key_pressed (GtkWidget *widget, GdkEvent *event)
{
	guint	action = DL_MENUITEM_INVALID;
	guint	key = event->key.keyval;
	if (key>='A' && key<='Z') key = key + ('a'-'A');	// to lower case
	last_time_of_user_click = time(NULL);
	switch (key)
	{
		case DELETE_KEY:	action = DL_MENUITEM_CANCEL_SELECTED;
					break;
		case 'p':		action = DL_MENUITEM_PAUSE_SELECTED;
					break;
		case 'r':		action = DL_MENUITEM_RESUME_SELECTED;
					break;
		case 'l':		action = DL_MENUITEM_SET_LOW_PRIO;
					break;
		case 'n':		action = DL_MENUITEM_SET_NORMAL_PRIO;
					break;
		case 'h':		action = DL_MENUITEM_SET_HIGH_PRIO;
					break;
		case 'u':		action = DL_MENUITEM_CLEAR_SELECTION;
					break;
		case 'a':		action = DL_MENUITEM_SELECT_ALL;
					break;
		default:		return FALSE;
	}
	downloads_popupmenu_item_selected (NULL, GUINT_TO_POINTER(action));
	return TRUE; // has been dealt with - if key is not in list we return false right away
}


// downloads_scroll_vertical
//
// The list has scrolled vertically - make sure we've got gaplists for all visible gaps

static void
downloads_scroll_vertical (GtkCList *clist, GtkScrollType scroll_type, gfloat pos, gpointer data)
{
	get_gaps_for_visible_downloads ();
}


/* create_scrolled_download_clist
 *
 * Creates the clist for the downloads and puts it in a scrolled window
 * Returns widget of scrolled window containing the clist
 *
 */

static GtkWidget *
downloads_page_create_scrolled_clist (void)
{
	gchar	**tabtitles = g_strsplit (	_(" file name| status|size |transferred |left |speed |time | priority|"
										  " avail |sources |last dl'ing |last 100% | hash"),
										N_("|"), 0);
	gint	  i;

	g_return_val_if_fail (tabtitles!=NULL,NULL);

	clist_downloads = gtk_clist_new_with_titles (NO_OF_DOWNLOAD_COLS, tabtitles);	/* 8 columns */
	gtk_signal_connect (GTK_OBJECT (clist_downloads), N_("click-column"),
			    GTK_SIGNAL_FUNC(downloads_clist_column_clicked), NULL);
	gtk_signal_connect (GTK_OBJECT (clist_downloads), N_("button_press_event"),
				GTK_SIGNAL_FUNC(downloads_button_pressed), GTK_OBJECT(clist_downloads));
	gtk_signal_connect (GTK_OBJECT (clist_downloads), N_("key_press_event"),
				GTK_SIGNAL_FUNC(downloads_key_pressed), GTK_OBJECT(clist_downloads));
//	gtk_signal_connect_after (GTK_OBJECT (clist_downloads), "key_press_event",
//				GTK_SIGNAL_FUNC(downloads_key_pressed_after), GTK_OBJECT(clist_downloads));
	gtk_signal_connect (GTK_OBJECT (clist_downloads), N_("resize-column"),
				GTK_SIGNAL_FUNC(downloads_column_resized), NULL);
	gtk_signal_connect (GTK_OBJECT (clist_downloads), N_("scroll-vertical"),
				GTK_SIGNAL_FUNC(downloads_scroll_vertical), NULL);

	downloadsclist_resort();

// 	0=file name, 1=status, 2=size, 3=transferred, 4=left, 5=speed, 6=time,
// 	7=priority, 8=avail, 9=sources, 10=last dl'ing, 11=last 100%, 12=hash
	gtk_clist_set_column_justification (DL_CLIST, 2, GTK_JUSTIFY_RIGHT);
	gtk_clist_set_column_justification (DL_CLIST, 3, GTK_JUSTIFY_RIGHT);
	gtk_clist_set_column_justification (DL_CLIST, 4, GTK_JUSTIFY_RIGHT);
	gtk_clist_set_column_justification (DL_CLIST, 5, GTK_JUSTIFY_RIGHT);
	gtk_clist_set_column_justification (DL_CLIST, 6, GTK_JUSTIFY_RIGHT);
	gtk_clist_set_column_justification (DL_CLIST, 8, GTK_JUSTIFY_RIGHT);
	gtk_clist_set_column_justification (DL_CLIST, 9, GTK_JUSTIFY_RIGHT);
	gtk_clist_set_column_justification (DL_CLIST,10, GTK_JUSTIFY_RIGHT);
	gtk_clist_set_column_justification (DL_CLIST,11, GTK_JUSTIFY_RIGHT);

	// set multi- or single- selection mode
	if (option_DLSingleSelectionMode==1)
	{
		gtk_clist_set_selection_mode(DL_CLIST, GTK_SELECTION_SINGLE);
	} else gtk_clist_set_selection_mode(DL_CLIST, GTK_SELECTION_EXTENDED);

	/* set columns resizable by user (which is not default in gtk2) */
	for (i=0; i<NO_OF_DOWNLOAD_COLS; i++)
	{
		gtk_clist_set_column_resizeable(DL_CLIST, i, TRUE);
	}

	// set column visibility as requested by user - hash INVISIBLE for now
	for (i=0; i<NO_OF_DOWNLOAD_COLS-1; i++) // not last column - hash!
		gtk_clist_set_column_visibility (DL_CLIST,i, (dlcols_visibility[i]==1));
	gtk_clist_set_column_visibility (DL_CLIST, NO_OF_DOWNLOAD_COLS-1, FALSE);

        // set default column widths
	set_clist_columns (DL_CLIST ,DL_COLS, NO_OF_DOWNLOAD_COLS);

	// create a scrollbar window to contain the clist later on

	dl_scrollwin = gtk_scrolled_window_new (NULL,NULL);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW(dl_scrollwin), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
	gtk_container_add (GTK_CONTAINER(dl_scrollwin), clist_downloads);
	gtk_widget_show (clist_downloads);
	gtk_widget_show (dl_scrollwin);

	g_strfreev(tabtitles);

	return (dl_scrollwin);
}


/* downloads_page_set_free_space
 *
 *
 */

void
downloads_page_set_free_space (gfloat needed, gfloat temp, gfloat incoming)
{
	gchar *stattxt = NULL;

	g_return_if_fail (downloads_page_stats_label!=NULL);

	if (needed == -1.0 && temp == -1.0 && incoming == -1.0)
	{
		gtk_label_set_text (GTK_LABEL(downloads_page_stats_label), _(" Downloads "));
		return;
	}

	stattxt = g_strdup_printf (_(" Downloads - need %.2fMB - free in temp: %.2fMB - "
							  	"free in incoming: %.2fMB "), needed, temp, incoming);
	g_return_if_fail (stattxt!=NULL);
	gtk_label_set_text (GTK_LABEL(downloads_page_stats_label), stattxt);
	g_free(stattxt);
}



static GtkWidget *
downloads_page_create_stats_label (void)
{
	GtkWidget	*hbox, *emptylabel;

	hbox = gtk_hbox_new(FALSE,0);
	gtk_widget_show(hbox);

	emptylabel = gtk_label_new(N_(" "));
	gtk_widget_show(emptylabel);

	downloads_page_stats_label = gtk_label_new (_(" Downloads "));
	gtk_label_set_justify (GTK_LABEL(downloads_page_stats_label), GTK_JUSTIFY_LEFT);
	gtk_widget_show(downloads_page_stats_label);

	gtk_box_pack_start (GTK_BOX(hbox), downloads_page_stats_label, FALSE, FALSE, 0);
	gtk_box_pack_start (GTK_BOX(hbox), emptylabel, TRUE, TRUE, 0);

	if (option_nogreen == 0)
		set_style_recursively (hbox, style_notebook_label);

	return hbox;
}


/* downloads_page_create
 *
 * Creates the downloads page of the notebook and returns it
 *
 */

GtkWidget *
downloads_page_create (void)
{
	GtkWidget	*vbox;

	vbox = gtk_vbox_new (FALSE,0);
	gtk_widget_show(vbox);

	g_return_if_fail (vbox!=NULL);

	gtk_box_pack_start (GTK_BOX(vbox), downloads_page_create_stats_label(), FALSE, FALSE, 0);
	gtk_box_pack_start (GTK_BOX(vbox), downloads_page_create_scrolled_clist(), TRUE, TRUE, 0);

	return vbox;
}

