/*
* global.h
*
* global variables and definitions
*
*/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef _ED2K_GUI_GLOBAL_H_INCLUDED_
#define _ED2K_GUI_GLOBAL_H_INCLUDED_

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#undef MIN	// these are defined in sys/param.h and in gtk/gtk.h ...
#undef MAX
#include <sys/param.h>
#include <time.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <gtk/gtk.h>

#ifdef FREEBSD
#warning *****************************
#warning * FreeBSD people:
#warning *****************************
#warning *****************************
#warning *****************************
#endif
#include <libintl.h>	/* for internationalisation with gettext */
#include <locale.h>

#ifdef FREEBSD
# define OP_SYSTEM_STRING "FreeBSD"
#else
# ifdef LINUX
#  define OP_SYSTEM_STRING "Linux"
# else
#  ifdef NETBSD
#   define OP_SYSTEM_STRING "NetBSD"
#  else
#   ifdef OPENBSD
#    define OP_SYSTEM_STRING "OpenBSD"
#   else
#    define OP_SYSTEM_STRING ""				/* some other unix/platform */
#   endif /* ifdef OPENBSD */
#  endif /* ifdef NETBSD */
# endif /* ifdef LINUX */
#endif /* ifdef FREEBSD */

#define MIN_CORE_VERSION 58

#define CORE_BUILD_1	0x02021c		// core from Feb 28, 2002

/* cores earlier than 2002-Dec-07 won't return search results */
#define OVERNET_CORE_TOO_OLD		0x020c07	
#define CORE_TOO_FAR_IN_THE_FUTURE	0x040000
#define CORE_JANUAR_2003		0x030101

#define CORE_IS_GAP_ENABLED	(core_build_date>=CORE_BUILD_1)

#define G_FREE(somepointer)	if(somepointer){g_free(somepointer);somepointer=NULL;}

// define keyvals for clist key press stuff
#define DELETE_KEY 65535
#define ENTER_KEY 65293
#define RSHIFT_KEY 65506
#define LSHIFT_KEY 65505

#ifdef _WIN32
#  define SLASH_CHAR '\\'
#  define SLASH_STRING "\\"
#else
#  define SLASH_CHAR '/'
#  define SLASH_STRING "/"
#endif

#  define MY_MAX_PATHFN_LEN	(MY_MAX_PATH_LEN+MY_MAX_FILENAME_LEN)


// Some macros to process a selection from a CList:
// The GList clist->selection contains the row numbers of the selected rows ...
//	(ie. care needs to be used if rows are removed while processing a selection list!)
#define PROCESS_CLIST_SELECTION_LIST_CUSTOM_LIST(clist,selection,filelist,castDatatype,block)\
			{\
				GList *items;\
				castDatatype *item;\
				g_return_if_fail(clist!=NULL);\
				items=selection;\
				while(items)\
				{\
					guint row;\
					gboolean item_is_in_filelist;\
					row=GPOINTER_TO_UINT(items->data);\
					item=(castDatatype*)gtk_clist_get_row_data(GTK_CLIST(clist),row);\
					item_is_in_filelist=FALSE;\
					/* only process item if it is found in the filelist */\
					if (filelist)\
					{\
						/* filelist should be a GPtrArray */\
						gint filelistitems = filelist->len;\
						gint filelistitemcounter;\
						for (filelistitemcounter=0; (!item_is_in_filelist) && filelistitemcounter<filelistitems; filelistitemcounter++)\
						{\
							gpointer filelistitempointer = g_ptr_array_index(filelist,filelistitemcounter);\
							if (filelistitempointer==item)\
								item_is_in_filelist=TRUE;\
						}\
					}\
					if (item_is_in_filelist)\
					{block}\
					items=items->next;\
				}\
			}

#define PROCESS_CLIST_SELECTION_LIST(clist,filelist,castDatatype,block)		\
			{\
				g_return_if_fail(clist!=NULL);\
				PROCESS_CLIST_SELECTION_LIST_CUSTOM_LIST(clist,clist->selection,filelist,castDatatype,block)\
			}

#define PROCESS_GLIST(glist,castToDatatype,block)	{GList *pgl_node=glist; while(pgl_node){ castToDatatype *item = (castToDatatype*) pgl_node->data; { block } pgl_node=pgl_node->next;}}
#define PROCESS_GSLIST(gslist,castToDatatype,block)	{GSList *pgl_node=gslist; while(pgl_node){ castToDatatype *item = (castToDatatype*) pgl_node->data; { block } pgl_node=pgl_node->next;}}

#define PROCESS_GPTRARRAY(parr,castToDatatype,block)		{if(parr){gint gpainc; for(gpainc=0; parr->len > gpainc; gpainc++){ castToDatatype *item = (castToDatatype*) g_ptr_array_index(parr,gpainc); { block }}}}
#define PROCESS_GPTRARRAY_FROM_END(parr,castToDatatype,block)	{if(parr){gint gpainc; for(gpainc=parr->len-1; gpainc>=0; gpainc--){ castToDatatype *item = (castToDatatype*) g_ptr_array_index(parr,gpainc); { block }}}}


#define UPDATE_INTERVAL 1000    /* how often notebook pages are updated... (in ms)      */

// For possible future use of gettext, so we can prepare the source code

#define	_(String)				gettext(String)	// strings that we might want to translate
/*
#define	_(String)				String		// strings that we might want to translate
*/
#define gettext_noop(String)	String
#define N_(String)				gettext_noop(String)	// strings that do not require translation

#define FIXME_STRING	N_("[FIXME]")

void	main_clean_up_connections_and_callbacks_before_quit (void);

#endif
