/*
*  gui_options.c
*
*  deals with loading and saving gui options
*
*  (c) Tim-Philipp Muller <t.i.m@orange.net>
*
*/

#include "gui_options.h"
#include "global.h"
#include "gaps.h"
#include "uploads_page.h"
#include "downloads_attrib.h"
#include "downloads_page.h"
#include "linkhandler.h"
#include "status_page.h"
#include "misc.h"
#include "stats.h"
#include "http_get.h"
#include "parse_message.h"


#include <time.h>
#include <stdio.h>
#include <gtk/gtk.h>
#include <sys/types.h>
#include <sys/stat.h>

// global variables

// options which are set & evaluated by donkey --- CORE ---

gboolean 	option_always_connected;	// flags: 1=true, 0=false
gboolean	option_verbose;
gboolean	option_save_corrupted;
gboolean    option_autoremove;              // automatically remove dead servers
gboolean	option_allowpm;			// allow private messages

gfloat	option_max_dl_speed = 9999.0;
gfloat	option_max_ul_speed = 9999.0;

char    option_incoming [260];
char    option_temporary[260];
char    option_nickname [50];

int     option_doorport = 0;
int     option_adminport = 0;

int     option_max_connections = 0;

int		option_line_down_speed = 0;
guint32	core_build_date = 0;

// options set and evaluated by --- GUI ---

/* have we notified the user of certain changes from v0.4.0 => v0.4.1? */
int 	option_change_message_1_shown = 0;

int		option_minfreetempspace = 15;	/* pause all downloads if space in temp is less than this many MB */
int		option_autoremoveinvalid = 1;	/* automatically remove local servers */
int     option_verify_cancels = 1;
int     option_verify_exit    = 1;
int     option_shutdown_core_on_exit = 1;
int     option_connect_make_def_values = 1;
int 	option_spawn_core_on_startup = 0;	/* if none is already running */
int 	option_connect_to_core_on_startup = 0;
int 	option_auto_clear_searchlist = 1;
int     option_show_own_files_in_search = 0;
int     option_server_rotate = 0;			// 0=off, >0 = how many minutes before disconnect+reconnect
int     option_fetch_when_too_few_servers = 1;
int     option_clear_slist_when_fetching_new_ones = 0;
int 	option_download_ed2k_link_immediately = 1; 	// don't add it to search list, start downloading right away
int 	option_restart_crashed_core = 1;		// try to respawn core on crash or kill
int 	option_disconnect_if_firewalled = 0;		// disconnect if server (wrongly) thinks that I'm firewalled
int 	option_color_whole_line_in_downloads=0;		// does what it says (yuk!)
int 	option_nofnjunk = 0;				// in DL list, tidy filenames of junk
int		option_keep_extensions = 0;			// keep extensions when tidying up DL list
int		option_disable_doubleclick = 0;		// otherwise mousewheel people might have a problem...
int		option_show_blacklist_items_grey = 0;	// instead of not showing them at all...
int		option_ignore_queue_deadlock = 0;	// don't try to break 'placed on connection queue' deadlocks
int		option_nogreen = 0;					// for those people who've got nifty gtk-styles
int		option_gnomeicons = 0;				// AXEL (Tim put default to 0 ;)
int		option_slowconnection =0;		// to minimise transfers on slow connections
int		option_noinsuffdspause=0;		// don't pause those downloads which have 'insuff.diskspace'
						//  when doing a conditional pause all X
int		option_clearstatuswindow=0;		// clear status window every x minutes (0=never)
int		option_logtimestamp=0;			// show timestamps in status window as well
int		option_nolinkstolog=0;			// don't output ed2k-links to statuslog
int		option_progressbar=0;			// colour progressbar as 'availability'?
int		option_greenshading=1;			// shade available parts in green (no=blue)
int		option_blacklistondl=0;			// add items to download automatically to blacklist
int		option_dltransinpercent=0;		// show 'transfered' in downloads in %
int		option_visibleonly=0;			// progressbar update: update only gaps for visible rows
int		option_noactivedlcancel=0;		// don't allow dl cancels w/o confirmation if dl is not paused
int		option_no_app_icon=-1;			// don't set the application icon (set 1 per -i cmd switch)

int		option_DLUnselectAfterAction = 0;	// unselect all downloads after each action
int		option_DLSingleSelectionMode = 0;	// put download clist in single selection mode
int		option_DLShowPartFileName = 1;		// show the x.part filename in the downloadlist with the filename

int		option_SRVUnselectAfterAction = 0;	// unselect all servers after each action
int		option_SRVSingleSelectionMode = 0;	// put server clist in single selection mode

gint	option_search_notebook_tabs_pos = (gint) GTK_POS_LEFT;
gint	option_main_notebook_tabs_pos = (gint) GTK_POS_TOP;
gint	option_main_notebook_tabs_hide_text = 1;

guint option_linkhandler_what_to_do_if_no_GUI_running = LINKHANDLER_IF_NO_GUI_ASK;

unsigned int	option_gui_instance = 0;	// set via the command line option '-n'

char    * option_defaulthost = NULL;		// host where the donkey client/core runs
int     option_defaultport;
char    * option_defaultuser = NULL;		// default core login username to show in connect dialog
char    * option_defaultpass = NULL;		// default core login password to show in connect dialog
char    * option_defaultdonkeypath = NULL;	// default path to donkey binary

char	* option_serverlist_URL = NULL;		// complete URL to serverlist html page with ed2k:// links
char	* option_serverlist_URL2 = NULL;	// complete URL to serverlist html page with ed2k:// links

char	* option_exec_on_complete = NULL;	// shell command to execute when a download is complete; %F=filename


gchar	*option_iconthemedir;				/* directory where the icons to use are */

guint	 option_maxlogfilesize = 250;		/* max. logfile size in kB */
guint	 option_show_extra_search_options = 1;	

// TODO: these two are overwritten on each connect() => also overwritten when doing http-fetch!
char    clienthost[513];                // actual host used (from connect dialog)
int     clientport;                     // actual port used (from connect dialog)

int     connect_right_away = 0;         /* using default options (don't bring up dialog at start */

char	* prefpath	= NULL;

char STANDARD_SERVERLIST_1[] = N_("http://www.edonkey2000.com/servers.html");
char STANDARD_SERVERLIST_2[] = N_("http://users.aber.ac.uk/tpm01/maxsrv.html");

/* window properties */

/* set these values generous, so we don't have too much incompatibility
*  between options files when adding new features
*/
int	column_widths [MAX_NO_OF_CLISTS][MAX_NO_OF_COLS] = {
				{ 200, 80, 80,100, 70, 70, 70, 70, 70, 70,70,70,0,0,0,0 },
				{ 220,120, 70,  0,  0,  0,  0,  0,  0,  0, 0, 0,0,0,0,0 },
				{ 130,  0,  0,  0,  0,  0,  0,  0,  0,  0, 0, 0,0,0,0,0 },
				{ 250,120, 90, 50, 50, 50, 50, 50, 50, 50,50, 0,0,0,0,0 },
				{ 200,140, 80, 80, 80, 80,  0,  0,  0,  0, 0, 0,0,0,0,0 },
				{ 200,  0,  0,  0,  0,  0,  0,  0,  0,  0, 0, 0,0,0,0,0 },
				{ 200,  0,  0,  0,  0,  0,  0,  0,  0,  0, 0, 0,0,0,0,0 },
				{ 150,150,300, 80,  0,  0,  0,  0,  0,  0, 0, 0,0,0,0,0 },
				{   0,  0,  0,  0,  0,  0,  0,  0,  0,  0, 0, 0,0,0,0,0 },
				{   0,  0,  0,  0,  0,  0,  0,  0,  0,  0, 0, 0,0,0,0,0 },
				{   0,  0,  0,  0,  0,  0,  0,  0,  0,  0, 0, 0,0,0,0,0 },
				{   0,  0,  0,  0,  0,  0,  0,  0,  0,  0, 0, 0,0,0,0,0 },
				{   0,  0,  0,  0,  0,  0,  0,  0,  0,  0, 0, 0,0,0,0,0 },
				{   0,  0,  0,  0,  0,  0,  0,  0,  0,  0, 0, 0,0,0,0,0 },
				{   0,  0,  0,  0,  0,  0,  0,  0,  0,  0, 0, 0,0,0,0,0 },
				{   0,  0,  0,  0,  0,  0,  0,  0,  0,  0, 0, 0,0,0,0,0 },
				};

int  lists_sorttype [MAX_NO_OF_CLISTS];		// init in read_gui_options
int  lists_sortcol  [MAX_NO_OF_CLISTS]; 	// init in read_gui_options
// which download list colums are visible?
int  dlcols_visibility  [MAX_NO_OF_COLS]; 	// init in read_gui_options

int  paned_win_position [MAX_NO_OF_PANED_WINS] = { 140,320,200,0,0,0,0,0,0,0,0,0,0,0,0,0 };
GtkAllocation	main_window_size;




// functions

static int		read_string_from_disk (char **p_str, FILE *infile);
static int		write_string_to_disk (char *s, FILE *outfile);




// takes filename and puts path where preferences files are stored in front of it
// flag_use_instances determines whether 'instance' appendix should be used on this file
// (e.g it should be used with 'gui_options', but not with 'gui_blacklist'
//
const gchar *
complete_pref_path (const gchar *fn, gboolean flag_use_instance)
{
	static gchar	*fullpath = NULL;
	gchar			 instance[15];
	
	g_return_val_if_fail(fn!=NULL, NULL);
	g_return_val_if_fail(prefpath!=NULL, NULL);
	
	if ((flag_use_instance) && option_gui_instance>0)
	{
		g_snprintf (instance, sizeof(instance), N_(".%u"), option_gui_instance);
	} else instance[0]=0x00;	// empty string (don't use index suffix)

	if (fullpath)
		g_free(fullpath);

	fullpath = g_strdup_printf (N_("%s%s%s%s"),
					prefpath,
					(prefpath[strlen(prefpath)-1]==SLASH_CHAR) ? N_("") : SLASH_STRING,
					fn,
					instance);

	g_return_val_if_fail(prefpath!=NULL, NULL);

	return fullpath;
}


// reads string from disk
// unsigned int: string len (chars)
// x chars: string (not terminated)
//
// returns -1 on error, otherwise string len (chars) read
//

#warning get rid of free() and malloc() here - use g_free etc.

static int
read_string_from_disk (char **p_str, FILE *infile)
{
	char *snew;
	unsigned int slen;

	g_return_val_if_fail (p_str!=NULL, -1);

	// assumption: the filelist item has been initialised => *pstr is not totally random!
	if (*p_str)
		free (*p_str);
	*p_str = NULL;
	// read stringlen	
	if (fread (&slen, 1, sizeof(unsigned int), infile)!=sizeof(unsigned int))
		return -1;
	if (slen>0)
	{
		// read string
		snew = (char*) malloc ((slen+1)*sizeof(char));
		if (!snew) return -1;
		memset (snew, 0x00, (slen+1)*sizeof(char));
		if (fread (snew, sizeof(char), slen, infile)!=(sizeof(char)*slen)) return -1;
	} else snew = strdup (N_(""));
	*p_str = snew;
	return slen;
}


// returns -1 on error, otherwise stringlen (chars) written
//
static int
write_string_to_disk (char *s, FILE *outfile)
{
	unsigned int slen=0;
	if (s) slen=strlen(s);
	// write stringlen
	fwrite (&slen, 1, sizeof(unsigned int), outfile);
	if (slen>0)
	{
		// write string
		if (fwrite (s, sizeof(char), slen, outfile) != (slen*sizeof(char)))
			return -1;
	}
	return slen;
}


/******************************************************************************
 *
 *   uploadstats_read_from_file
 *
 *   reads the upload stats from disk (the new ascii format version)
 *
 */

void
uploadstats_read_from_file (void)
{
	gchar **line, **linearray;

	/* init this properly in case file isn't there */
	stats_last_cleared = time(NULL);

	// Now read in the file
	linearray = misc_get_array_of_lines_from_textfile (complete_pref_path (N_("gui_uploadstats.txt"), GUI_CONFIG_USE_INSTANCE));
	if (linearray)	// there should be at least our header!
	{
		guint vermaj, vermin, verminmin;
		line = linearray;
		if ((*line) && sscanf(*line, N_("# ed2k_gui --- gui_uploadstats --- config format version %u.%u.%u\n"), &vermaj, &vermin, &verminmin)==3)
		{
			gchar			 hashstr[33], fn[4096];
			time_t			 last_upload;
			guint32			 size;
			gfloat			 uploaded;

			line++;
			while (*line)
			{
				if (**line!='#' && **line!=0x00)
				{
					fn[0]=0x00;
					if (5 == sscanf(*line, N_("%32s %u %f %lu \\%4095[^\\]\\\n"), hashstr, &size, &uploaded, &last_upload, fn))
					{
						uploadRecord *ul = g_new0 (uploadRecord, 1);
						g_return_if_fail (ul!=NULL);
						ul->namequark = g_quark_from_string(fn);
						ul->size = size;
						ul->speed = 0.0;
						ul->username = N_("");	// doesn't matter here
						memcpy (ul->hash, hash_str_to_hash(hashstr), 16);
						uploadstats_add_or_increase_record (ul, 0);
						if (uploadstats)
						{
							uploadstatsRecord *ulstatsr = g_ptr_array_index (uploadstats, uploadstats->len -1);
							if (ulstatsr)
							{
								ulstatsr->size = ul->size;
								ulstatsr->last_uploaded = last_upload;
								ulstatsr->uploaded = uploaded;
							}
						}
						g_free(ul);	// don't call upload_free_record(), 'cause username is static here
					}
				}
				line++;
			}
		} else g_printerr (_("gui_uploadstats.txt: invalid header?\n"));
		g_strfreev (linearray);
	} else g_printerr (_("gui_uploadstats.txt: couldn't read file or file empty.\n"));

	stats_read_from_disk ();
}



void
write_gui_uploadstats (void)
{
	FILE		*f;
	const gchar	*filename = complete_pref_path (N_("gui_uploadstats.txt"), GUI_CONFIG_USE_INSTANCE);

	g_return_if_fail ((filename));

	if ((f = fopen(filename, N_("w")) )!=NULL)
	{
		fprintf (f, N_("# ed2k_gui --- gui_uploadstats --- config format version 0.3.0\n"));
		fprintf (f, N_("#\n%s\n"), _("# This file is created by the ed2k gui. It contains your"));
		fprintf (f, N_("%s\n#\n"), _("# upload statistics. DO NOT CHANGE THE ABOVE HEADER!"));
		fprintf (f, N_("%s"), _("# file hash - file size - amount uploaded - time last uploaded - file name\n"));

		if (uploadstats)
		{
			PROCESS_GPTRARRAY ( uploadstats, uploadstatsRecord,
				if (item)
				{
					const gchar *fn = g_quark_to_string (item->namequark);
					if (!fn) fn = N_("???");
					fprintf (f, N_("%32s %u %f %lu \\%s\\\n"),
						hash_to_hash_str(item->hash),
						item->size,
						item->uploaded,
						item->last_uploaded,
						fn);
				}
			);
		}

		// set file permissions so that only the owner can read and write to it
		fchmod (fileno(f), (mode_t) S_IRUSR|S_IWUSR);

		fclose (f);
        } else status_system_error_msg(_("couldn't save upload stats"));
		stats_save_to_disk ();
}

// checks if a URL is unwanted and corrects that if the case
//
// returns
// -1 if error or URL not okay
//  0 if URL okay + not changed
//  1 if URL okay + changed
//
int check_for_unwanted_URL (char **urlstr)
{
	if (!urlstr) return -1;
	if ((!*urlstr) || ((*urlstr) && strncmp(*urlstr, N_("http://bse.dnsalias.org/ed2k/serverlist.html"),28)==0))
	{
		strrealloccpy (urlstr, STANDARD_SERVERLIST_2);
		return 1;
	}
	return 0;
}

void read_gui_options (void)
{
	FILE		*of;
	const gchar	*filename	= complete_pref_path (N_("gui_options"), GUI_CONFIG_USE_INSTANCE);
	char		*stringbuf	= (char*) alloca(2048*sizeof(char));
	int	  		i, dummy;
	char	  	use_new_string_scheme = 0x00;

	g_return_if_fail ((filename));

	// set some default GUI options in case the gui_options file does not exist

	for (i=0; i<MAX_NO_OF_CLISTS; i++)
	{
		lists_sorttype[i] = GTK_SORT_DESCENDING;
		lists_sortcol[i]  = 0;
	}
	for (i=0; i<MAX_NO_OF_COLS; i++)
	{
		dlcols_visibility[i] = 1;
	}
	dlcols_visibility[NO_OF_DOWNLOAD_COLS-1] = 0;	// not the hash

	option_verify_cancels	     = 1;
	option_verify_exit           = 1;
        option_shutdown_core_on_exit = 0;	// nope

        option_defaultport      = 4663;

	strrealloccpy (&option_defaulthost, N_("localhost"));

	strrealloccpy (&option_defaultuser, N_(""));
	strrealloccpy (&option_defaultpass, N_(""));
    strrealloccpy (&option_defaultdonkeypath, N_(""));
	strrealloccpy (&option_exec_on_complete, N_(""));

	strrealloccpy (&option_serverlist_URL, STANDARD_SERVERLIST_1);
	strrealloccpy (&option_serverlist_URL2, STANDARD_SERVERLIST_2);

	main_window_size.height = 480;
	main_window_size.width  = 640;
	main_window_size.x	= X_POS_CORRECT;
	main_window_size.y	= Y_POS_CORRECT;

	// now read in options
	/* XXX - Don't flatten the hierarchy here with a return w/o doing the
	 * check if the pixmap path is set from below...!
	 */
	if ((of = fopen(filename, N_("rb")) )!=NULL)
	{
		fread ( column_widths, MAX_NO_OF_CLISTS*MAX_NO_OF_COLS, sizeof(int), of);
		fread ( paned_win_position, MAX_NO_OF_PANED_WINS, sizeof(int), of);
		fread ( &main_window_size, sizeof(GtkAllocation), 1, of);
		fread ( &option_verify_cancels,         1, sizeof(int),of);
		fread ( &option_verify_exit,            1, sizeof(int),of);
		fread ( &option_shutdown_core_on_exit,  1, sizeof(int),of);
		
		// maintain backwards compatibility for these strings...
		
		memset (stringbuf, 0x00, 2048*sizeof(char));
		fread  (stringbuf, 1, 128, of);	// NB: _not_ *sizeof(char) here!
		strrealloccpy (&option_defaulthost, stringbuf);
// OLD:		fread ( option_defaulthost, 1, sizeof(option_defaulthost), of);
		
                fread ( &option_defaultport,            1, sizeof(int),of);
		
		memset (stringbuf, 0x00, 2048*sizeof(char));
		fread  (stringbuf, 1, 128, of);	// NB: _not_ *sizeof(char) here!
		strrealloccpy (&option_defaultuser, stringbuf);
// OLD:		fread ( option_defaultuser, 1, sizeof(option_defaultuser), of);

		memset (stringbuf, 0x00, 2048*sizeof(char));
		fread  (stringbuf, 1, 128, of);	// NB: _not_ *sizeof(char) here!
		strrealloccpy (&option_defaultpass, stringbuf);
// OLD:		fread ( option_defaultpass, 1, sizeof(option_defaultpass), of);

		memset (stringbuf, 0x00, 2048*sizeof(char));
		fread  (stringbuf, 1, 512, of);	// NB: _not_ *sizeof(char) here!
		strrealloccpy (&option_defaultdonkeypath, stringbuf);
// OLD:		fread ( option_defaultdonkeypath, 1, sizeof(option_defaultdonkeypath), of);

                fread ( &option_connect_make_def_values,1, sizeof(int),of);
                fread ( &option_spawn_core_on_startup,  1, sizeof(int),of);
                fread ( &option_connect_to_core_on_startup,1, sizeof(int),of);
		fread ( &option_auto_clear_searchlist,  1, sizeof(int),of);
		fread ( &option_show_own_files_in_search,1,sizeof(int),of);
		fread ( lists_sorttype, MAX_NO_OF_CLISTS,  sizeof(int),of);
		fread ( lists_sortcol,  MAX_NO_OF_CLISTS,  sizeof(int),of);
		fread ( &option_server_rotate,          1, sizeof(int),of);
		
		memset (stringbuf, 0x00, 2048*sizeof(char));
		fread  (stringbuf, 1, 512, of);	// NB: _not_ *sizeof(char) here!
		strrealloccpy (&option_serverlist_URL, stringbuf);
// OLD:		fread ( option_serverlist_URL, 1, sizeof(option_serverlist_URL), of);
		
		fread ( &option_clear_slist_when_fetching_new_ones, 1, sizeof(int), of);
		
		memset (stringbuf, 0x00, 2048*sizeof(char));
		fread  (stringbuf, 1, 512, of);	// NB: _not_ *sizeof(char) here!
		strrealloccpy (&option_serverlist_URL2, stringbuf);
// OLD:		fread ( option_serverlist_URL2, 1, sizeof(option_serverlist_URL2), of);

		fread ( &dummy, 1, sizeof(int), of);
		(void) toolbar_set_cons (dummy);
		
		memset (stringbuf, 0x00, 2048*sizeof(char));
		fread  (stringbuf, 1, 513, of);	// NB: _not_ *sizeof(char) here!
		strrealloccpy (&option_exec_on_complete, stringbuf);
// OLD:		fread ( option_exec_on_complete, 1, sizeof(option_exec_on_complete), of);

		/* option_use_small_logo: DEPRECATED */
		fread ( &dummy, 1, sizeof(int), of);

		fread ( &option_download_ed2k_link_immediately, 1, sizeof(int), of);
		fread ( &option_fetch_when_too_few_servers, 1, sizeof(int), of);
		fread ( &option_restart_crashed_core, 1, sizeof(int), of);
		fread ( &option_disconnect_if_firewalled, 1, sizeof(int), of);
		fread ( &option_color_whole_line_in_downloads, 1, sizeof(int), of);
		fread ( &option_nofnjunk, 1, sizeof(int), of);
		fread ( dlcols_visibility, MAX_NO_OF_COLS, sizeof(int), of);
		fread ( &option_keep_extensions, 1, sizeof(int), of);
		fread ( &option_disable_doubleclick, 1, sizeof(int), of);
		fread ( &option_show_blacklist_items_grey, 1, sizeof(int), of);
		fread ( &option_ignore_queue_deadlock, 1, sizeof(int), of);
		fread ( &option_nogreen, 1, sizeof(int), of);
		fread ( &option_slowconnection, 1, sizeof(int), of);
		fread ( &option_clearstatuswindow, 1, sizeof(int), of);
		fread ( &option_noinsuffdspause, 1, sizeof(int), of);
		fread ( &option_logtimestamp, 1, sizeof(int), of);
		fread ( &option_nolinkstolog, 1, sizeof(int), of);
		fread ( &option_progressbar, 1, sizeof(int), of);
		fread ( &option_greenshading, 1, sizeof(int), of);
		fread ( &option_blacklistondl, 1, sizeof(int), of);
		fread ( &option_dltransinpercent, 1, sizeof(int), of);
		fread ( &option_visibleonly, 1, sizeof(int), of);
		if (option_no_app_icon==-1)
		{	// has not been set by command line => get status from options file
			fread ( &option_no_app_icon, 1, sizeof(int), of);
			if (option_no_app_icon==-1) option_no_app_icon=0;	// if old gui..
		} else fread ( &dummy, 1, sizeof(int), of);	// don't overwrite new set value
		fread ( &option_noactivedlcancel, 1, sizeof(int), of);
		fread ( &use_new_string_scheme, 1, sizeof(char), of);
		
		if (use_new_string_scheme!=0x00)
		{
			read_string_from_disk ( & option_defaulthost, of);
			read_string_from_disk ( & option_defaultuser, of);
			read_string_from_disk ( & option_defaultpass, of);
			read_string_from_disk ( & option_defaultdonkeypath, of);
			read_string_from_disk ( & option_serverlist_URL, of);
			read_string_from_disk ( & option_serverlist_URL2, of);
			read_string_from_disk ( & option_exec_on_complete, of);

			fread ( &option_DLUnselectAfterAction, 1, sizeof(int), of);
			fread ( &option_DLSingleSelectionMode, 1, sizeof(int), of);
			fread ( &option_DLShowPartFileName, 1, sizeof(int), of);
			fread ( &option_SRVUnselectAfterAction, 1, sizeof(int), of);
			fread ( &option_SRVSingleSelectionMode, 1, sizeof(int), of);

			fread ( &option_search_notebook_tabs_pos, 1, sizeof(gint), of);
			fread ( &option_main_notebook_tabs_pos, 1, sizeof(gint), of);
			fread ( &option_main_notebook_tabs_hide_text, 1, sizeof(gint), of);
			/* option_show_edonkey_logo: deprecated */
			fread ( &dummy, 1, sizeof(gint), of);

			fread ( &option_timeline_active, 1, sizeof(gint), of);
			option_timeline_active = option_timeline_active % STATS_TIMELINE_LAST;
			fread ( &option_timeline_mode, 1, sizeof(gint), of);
			option_timeline_mode = option_timeline_mode % STATS_TIMELINE_DISPLAYMODE_LAST;

			fread ( &option_stats_up_wrong, 1, sizeof(gfloat), of);
			fread ( &option_stats_down_wrong, 1, sizeof(gfloat), of);
			fread ( &option_stats_up_crop, 1, sizeof(gfloat), of);
			fread ( &option_stats_down_crop, 1, sizeof(gfloat), of);

			/* DEPRECATED */
			fread (&option_gnomeicons, 1, sizeof(int), of);	// AXEL
			option_gnomeicons = 0;

			(void) read_string_from_disk (&option_iconthemedir, of);

			fread ( &option_show_extra_search_options, 1, sizeof(gint), of);
			fread ( &option_minfreetempspace, 1, sizeof(int), of);
			fread ( &option_autoremoveinvalid, 1, sizeof(int), of);
			fread ( &option_change_message_1_shown, 1, sizeof(int), of);

			fread ( &option_linkhandler_what_to_do_if_no_GUI_running, 1, sizeof(guint),of);
			
			// read more options...
//			fread ( &, 1, sizeof(gint), of);

		}
		fclose (of);
	}

	// verify some restrictions are met

	if ((!option_iconthemedir) || *option_iconthemedir==0x00)
		option_iconthemedir = g_strdup_printf (N_("%s/default"), APP_PIXMAPS);

	/* If APP_PIXMAPS starts with '/usr/share', then we probably habe a binary package.
	 * If the person compiled the GUI from source before and used it, then the pixmap path
	 * will be set to /usr/local/share and thus new icons might not be loaded. We don't want
	 * that. Equally, if APP_PIXMAPS starts with '/usr/local/share' and the path configured
	 * is /usr/share, then it's probably a newer compile from source that supersedes the
	 * package. In both cases, we want to change the icon path in 99% of the cases.
	 */
	if (g_strncasecmp(APP_PIXMAPS, N_("/usr/share/"), 10 )==0)
	{
		if (g_strncasecmp(option_iconthemedir, N_("/usr/local/share/"), 16)==0)
		{
			gchar *newdir = g_strdup_printf (N_("%s/%s"), APP_PIXMAPS, g_basename(option_iconthemedir));
			/* leaks, but is safer for now */
			option_iconthemedir = newdir;
		}
	}

	if (g_strncasecmp(APP_PIXMAPS, N_("/usr/local/share/"), 16 )==0)
	{
		if (g_strncasecmp(option_iconthemedir, N_("/usr/share/"), 10)==0)
		{
			gchar *newdir = g_strdup_printf (N_("%s/%s"), APP_PIXMAPS, g_basename(option_iconthemedir));
			/* leaks, but is safer for now */
			option_iconthemedir = newdir;
		}
	}


	if (option_server_rotate>0 && option_server_rotate<30)
	{
		option_server_rotate=30;
	}
	check_for_unwanted_URL ( &option_serverlist_URL  );
	check_for_unwanted_URL ( &option_serverlist_URL2 );	

	// make upgrade from older version more smoothly
	if (column_widths[DL_COLS][10]==0) column_widths[DL_COLS][10]=70;
	if (column_widths[DL_COLS][11]==0) column_widths[DL_COLS][11]=70;

	if (column_widths[FSHARED_COLS][1]==0) column_widths[FSHARED_COLS][1]=70;
	if (column_widths[FSHARED_COLS][2]==0) column_widths[FSHARED_COLS][2]=70;
	if (column_widths[FSHARED_COLS][3]==0) column_widths[FSHARED_COLS][3]=70;
}

void write_gui_options (void)
{
	FILE		*of;
	const gchar	*filename	= complete_pref_path (N_("gui_options"), GUI_CONFIG_USE_INSTANCE);
	char		*strbuf	= (char*) alloca(514);	// NB: _not_ *sizeof(char)
	char		 use_new_string_scheme;
	guint		 dummy;

    g_return_if_fail (filename!=NULL);
	
	if ((of = fopen(filename, N_("wb")) )!=NULL)
	{
		// set file permissions so that only the owner can read and write to it
		fchmod (fileno(of), (mode_t) S_IRUSR|S_IWUSR);
	
		fwrite ( column_widths, MAX_NO_OF_CLISTS*MAX_NO_OF_COLS, sizeof(int), of);
		fwrite ( paned_win_position, MAX_NO_OF_PANED_WINS, sizeof(int), of);
		fwrite ( &main_window_size, sizeof(GtkAllocation), 1, of);
		fwrite ( &option_verify_cancels, 1, sizeof(int), of);
		fwrite ( &option_verify_exit,    1, sizeof(int), of);
		fwrite ( &option_shutdown_core_on_exit,  1, sizeof(int),of);
		
		// we want to maintain backwards compatibility...
		memset (strbuf, 0x00, 514);
		if (option_defaulthost) strncpy (strbuf, option_defaulthost, 513);
		fwrite ( strbuf, 1, 128, of);	// NB: _not_ *sizeof(char)
		
		fwrite ( &option_defaultport,    1, sizeof(int), of);
		
		memset (strbuf, 0x00, 514);
		if (option_defaultuser) strncpy (strbuf, option_defaultuser, 513);
		fwrite ( strbuf, 1, 128, of);	// NB: _not_ *sizeof(char)
		
		memset (strbuf, 0x00, 514);
		if (option_defaultpass) strncpy (strbuf, option_defaultpass, 513);
		fwrite ( strbuf, 1, 128, of);	// NB: _not_ *sizeof(char)
		
		memset (strbuf, 0x00, 514);
		if (option_defaultdonkeypath) strncpy (strbuf, option_defaultdonkeypath, 513);
		fwrite ( strbuf, 1, 512, of);	// NB: _not_ *sizeof(char)
		
		fwrite ( &option_connect_make_def_values,1, sizeof(int),of);
		fwrite ( &option_spawn_core_on_startup,1, sizeof(int),of);
		fwrite ( &option_connect_to_core_on_startup,1, sizeof(int),of);
		fwrite ( &option_auto_clear_searchlist,  1, sizeof(int),of);
		fwrite ( &option_show_own_files_in_search,1,sizeof(int),of);
		fwrite ( lists_sorttype, MAX_NO_OF_CLISTS,  sizeof(int),of);
		fwrite ( lists_sortcol,  MAX_NO_OF_CLISTS,  sizeof(int),of);
		fwrite ( &option_server_rotate,          1, sizeof(int),of);
		
		memset (strbuf, 0x00, 514);
		if (option_serverlist_URL) strncpy (strbuf, option_serverlist_URL, 513);
		fwrite ( strbuf, 1, 512, of);	// NB: _not_ *sizeof(char)
		
		fwrite ( &option_clear_slist_when_fetching_new_ones, 1, sizeof(int), of);
		
		memset (strbuf, 0x00, 514);
		if (option_serverlist_URL2) strncpy (strbuf, option_serverlist_URL2, 513);
		fwrite ( strbuf, 1, 512, of);	// NB: _not_ *sizeof(char)
		
		dummy = toolbar_set_cons (0);	/* just query max. no. of connections seen */
		fwrite ( &dummy, 1, sizeof(int), of);
		
		memset (strbuf, 0x00, 514);
		if (option_exec_on_complete) strncpy (strbuf, option_exec_on_complete, 513);
		fwrite ( strbuf, 1, 513, of);	// NB: _not_ *sizeof(char)
		
		/* option_use_small_logo: DEPRECATED */
		dummy = 0;
		fwrite ( &dummy, 1, sizeof(int), of);

		fwrite ( &option_download_ed2k_link_immediately, 1, sizeof(int), of);
		fwrite ( &option_fetch_when_too_few_servers, 1, sizeof(int), of);
		fwrite ( &option_restart_crashed_core, 1, sizeof(int), of);
		fwrite ( &option_disconnect_if_firewalled, 1, sizeof(int), of);
		fwrite ( &option_color_whole_line_in_downloads, 1, sizeof(int), of);
		fwrite ( &option_nofnjunk, 1, sizeof(int), of);
		fwrite ( dlcols_visibility, MAX_NO_OF_COLS, sizeof(int), of);
		fwrite ( &option_keep_extensions, 1, sizeof(int), of);
		fwrite ( &option_disable_doubleclick, 1, sizeof(int), of);
		fwrite ( &option_show_blacklist_items_grey, 1, sizeof(int), of);
		fwrite ( &option_ignore_queue_deadlock, 1, sizeof(int), of);
		fwrite ( &option_nogreen, 1, sizeof(int), of);
		fwrite ( &option_slowconnection, 1, sizeof(int), of);
		fwrite ( &option_clearstatuswindow, 1, sizeof(int), of);
		fwrite ( &option_noinsuffdspause, 1, sizeof(int), of);
		fwrite ( &option_logtimestamp, 1, sizeof(int), of);
		fwrite ( &option_nolinkstolog, 1, sizeof(int), of);
		fwrite ( &option_progressbar, 1, sizeof(int), of);
		fwrite ( &option_greenshading, 1, sizeof(int), of);
		fwrite ( &option_blacklistondl, 1, sizeof(int), of);
		fwrite ( &option_dltransinpercent, 1, sizeof(int), of);
		fwrite ( &option_visibleonly, 1, sizeof(int), of);
		fwrite ( &option_no_app_icon, 1, sizeof(int), of);
		fwrite ( &option_noactivedlcancel, 1, sizeof(int), of);
		// flag: use new string scheme
		use_new_string_scheme = 0x01;	
		fwrite ( &use_new_string_scheme, 1, sizeof(char), of);
		// write strings in new scheme
		write_string_to_disk ( option_defaulthost, of);
		write_string_to_disk ( option_defaultuser, of);
		write_string_to_disk ( option_defaultpass, of);
		write_string_to_disk ( option_defaultdonkeypath, of);
		write_string_to_disk ( option_serverlist_URL, of);
		write_string_to_disk ( option_serverlist_URL2, of);
		write_string_to_disk ( option_exec_on_complete, of);
		
		fwrite ( &option_DLUnselectAfterAction, 1, sizeof(int), of);
		fwrite ( &option_DLSingleSelectionMode, 1, sizeof(int), of);
		fwrite ( &option_DLShowPartFileName, 1, sizeof(int), of);
		fwrite ( &option_SRVUnselectAfterAction, 1, sizeof(int), of);
		fwrite ( &option_SRVSingleSelectionMode, 1, sizeof(int), of);

		fwrite ( &option_search_notebook_tabs_pos, 1, sizeof(gint), of);
		fwrite ( &option_main_notebook_tabs_pos, 1, sizeof(gint), of);
		fwrite ( &option_main_notebook_tabs_hide_text, 1, sizeof(int), of);
		/* option_show_edonkey_logo: deprecated */
		dummy = 0;
		fwrite ( &dummy, 1, sizeof(int), of);
		fwrite ( &option_timeline_active, 1, sizeof(int), of);
		fwrite ( &option_timeline_mode, 1, sizeof(int), of);

		fwrite ( &option_stats_up_wrong, 1, sizeof(gfloat), of);
		fwrite ( &option_stats_down_wrong, 1, sizeof(gfloat), of);
		fwrite ( &option_stats_up_crop, 1, sizeof(gfloat), of);
		fwrite ( &option_stats_down_crop, 1, sizeof(gfloat), of);

		/* DEPRECATED */
		option_gnomeicons = 0;
		fwrite (&option_gnomeicons, 1, sizeof(int), of);	// AXEL

		(void) write_string_to_disk (option_iconthemedir, of);

		fwrite ( &option_show_extra_search_options, 1, sizeof(gint), of);
		fwrite ( &option_minfreetempspace, 1, sizeof(int), of);
		fwrite ( &option_autoremoveinvalid, 1, sizeof(int), of);
		fwrite ( &option_change_message_1_shown, 1, sizeof(int), of);
		
		fwrite ( &option_linkhandler_what_to_do_if_no_GUI_running, 1, sizeof(guint),of);

		// write additional options...
//		fwrite ( &, 1, sizeof(int), of);

		fclose (of);
	} else status_system_error_msg(_("couldn't save options "));
}

void set_clist_columns (GtkCList *clist, int list_no, int no_of_cols)
{
	int i;
	for (i=0; i<no_of_cols; i++)
	{
		gtk_clist_set_column_width (clist, i, column_widths[list_no][i]);
	}
}

void remember_clist_columns (GtkCList *clist, int list_no)
{
	int i;
	for (i=0; i<MAX_NO_OF_COLS; i++)
	{
		/* = gtk_clist_get_column_width (clist, i);*/
  		column_widths[list_no][i] = (clist->column[i]).width;
	}
}


