/* icons.c
*
* exports icons
*
*/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "downloads_page.h"
#include "global.h"
#include "gui_options.h"
#include "icons.h"
#include "misc.h"
#include "status_page.h"

#include <sys/types.h>	/* for opendir() etc. */
#include <dirent.h>

#include <string.h>
#include <unistd.h>		/* for access() */

gchar *icon_none[] =
{
/* columns rows colors chars-per-pixel */
N_("22 22 1 1"),
N_("o c None"),
/* pixels */
N_("oooooooooooooooooooooo"),
N_("oooooooooooooooooooooo"),
N_("oooooooooooooooooooooo"),
N_("oooooooooooooooooooooo"),
N_("oooooooooooooooooooooo"),
N_("oooooooooooooooooooooo"),
N_("oooooooooooooooooooooo"),
N_("oooooooooooooooooooooo"),
N_("oooooooooooooooooooooo"),
N_("oooooooooooooooooooooo"),
N_("oooooooooooooooooooooo"),
N_("oooooooooooooooooooooo"),
N_("oooooooooooooooooooooo"),
N_("oooooooooooooooooooooo"),
N_("oooooooooooooooooooooo"),
N_("oooooooooooooooooooooo"),
N_("oooooooooooooooooooooo"),
N_("oooooooooooooooooooooo"),
N_("oooooooooooooooooooooo"),
N_("oooooooooooooooooooooo"),
N_("oooooooooooooooooooooo"),
N_("oooooooooooooooooooooo")
};

 gchar **icon_mainlogo = icon_none;
 gchar **icon_mainlogo_overnet = icon_none;
 gchar **icon_app = icon_none;
 gchar **icon_resume = icon_none;
 gchar **icon_resumeall = icon_none;
 gchar **icon_pause = icon_none;
 gchar **icon_pauseall = icon_none;
 gchar **icon_options = icon_none;
 gchar **icon_connect = icon_none;
 gchar **icon_cancel = icon_none;
 gchar **icon_search = icon_none;
 gchar **icon_folder = icon_none;
 gchar **icon_status = icon_none;
 gchar **icon_upload = icon_none;
 gchar **icon_download = icon_none;
 gchar **icon_servers = icon_none;
 gchar **icon_about = icon_none;
 gchar **icon_stats = icon_none;
 gchar **icon_threestars = icon_none;
 gchar **icon_twostars = icon_none;
 gchar **icon_onestar = icon_none;
 gchar **icon_extend = icon_none;
 gchar **icon_200more = icon_none;
 gchar **dot_yellow = icon_none;
 gchar **dot_white = icon_none;
 gchar **dot_red = icon_none;
 gchar **dot_purple = icon_none;
 gchar **dot_green = icon_none;
 gchar **dot_blue = icon_none;
 gchar **icon_updown = icon_none;
 gchar **icon_columnhide = icon_none;
 gchar **icon_columnshow = icon_none;
 gchar **icon_skull = icon_none;
 gchar **icon_ok = icon_none;
 gchar **icon_refresh = icon_none;
 gchar **icon_wizard = icon_none;
 gchar **icon_wizard_off = icon_none;
 gchar **icon_exit = icon_none;
 gchar **icon_disconnect = icon_none;
 gchar **icon_faq = icon_none;
 gchar **icon_mail = icon_none;
 gchar **icon_tux = icon_none;
 gchar **icon_ignition = icon_none;
 gchar **icon_ignition_small = icon_none;
 gchar **icon_core = icon_none;
 gchar **icon_eyes = icon_none;
 gchar **icon_optionon = icon_none;
 gchar **icon_optionoff = icon_none;
 gchar **icon_connections = icon_none;

 gchar **icon_gnome1 = icon_none;
 gchar **icon_gnome2 = icon_none;
 gchar **icon_kde = icon_none;

// global variables
GdkPixmap *dotyellow_pm, *dotwhite_pm, *dotred_pm, *dotpurple_pm, *dotgreen_pm, *dotblue_pm;
GdkBitmap *dotyellow_bm, *dotwhite_bm, *dotred_bm, *dotpurple_bm, *dotgreen_bm, *dotblue_bm;

// functions

/*
*  create_widget_from_xpm
*
*  convert an xpm string into a pixmap widget
*  uses win_main which is a global
*
*  function written by Eric Harlow
*/

GtkWidget *
create_widget_from_xpm (GtkWidget *win_widget, gchar **xpm_data)
{
	GdkBitmap	*mask;
	GdkPixmap	*pixmap_data;
	GtkWidget	*pixmap_widget;

	g_return_val_if_fail (win_widget!=NULL,NULL);
	g_return_val_if_fail (xpm_data!=NULL,NULL);

	// convert string to a gdk_pixmap - Note: window = main window widget
	pixmap_data = gdk_pixmap_create_from_xpm_d ( win_widget->window, &mask,
						     NULL, (gchar **) xpm_data);

	// convert the pixmap to a pixmap widget
	pixmap_widget = gtk_pixmap_new (pixmap_data, mask);

	gtk_widget_show (pixmap_widget);

	return (pixmap_widget);
}




static  gchar **
pixmap_read_from_file (const gchar *pixmapname)
{
	gchar	**arr;
	gchar	 *filename = g_strdup_printf("%s%c%s", option_iconthemedir, SLASH_CHAR, pixmapname);
	gchar	**ret = (gchar**) icon_none;

	g_return_val_if_fail (pixmapname!=NULL, NULL);
	g_return_val_if_fail (filename!=NULL, NULL);

	arr = misc_get_array_of_lines_from_textfile (filename);
	if (arr)
	{
		gchar *oneline, **line;

		line = arr;
		while (*line)
		{
			/* remove leading tabs and whitespaces in each line */
			while (**line==' ' || **line=='\t')
			{
				gint i;
				for (i=0; i<strlen(*line); i++)
				{
					gchar *l = *line;
					l[i] = l[i+1];
				}
			}
			line++;
		}

		oneline = g_strjoinv (N_(""), arr);
		if (g_strncasecmp(oneline,N_("/* XPM */"),9)==0)
		{
			gchar *start = strstr (oneline, N_("{\""));
			gchar *end = strstr (oneline, N_("\"};"));
			if ((start)&&(end))
			{
				start += 2;	// skrip bracket and opening quote
				*end = 0x00;

				ret = g_strsplit(start, N_("\",\""), 0);
				if (!ret)
					g_printerr (_("Pixmap %s - couldn't do g_strsplit()\n"), filename);
			} else g_printerr (_("Error loading pixmap '%s' - not XPM format?\n"), filename);
		} else g_printerr (_("Error loading pixmap '%s' - not XPM format? ['/* XPM */' missing]\n"), filename);

/*		{
			gchar **line = ret;
			g_print ("----------------------------------------------------------------\n");
			while (*line) { g_print ("%s\n", *line); line++; }
			g_print ("----------------------------------------------------------------\n");
		}
*/

		if (oneline) g_free(oneline);
		g_strfreev(arr);
	} else g_printerr (_("couldn't load pixmap '%s' - file doesn't exist or is empty.\n"), filename);
	if (filename) g_free(filename);
	return ret;
}

/*
static GdkPixmap *
pixmap_create_from_xpm_file (GtkWidget *widget, GdkBitmap **bm, const gchar *filename)
{
	gchar		**xpmarray = NULL;
	GdkPixmap	 *pm = NULL;

	g_return_val_if_fail (widget!=NULL, NULL);
	g_return_val_if_fail (bm!=NULL, NULL);
	g_return_val_if_fail (filename!=NULL, NULL);

	xpmarray = (gchar**) pixmap_read_from_file (filename);
	if (xpmarray)
	{
		pm = gdk_pixmap_create_from_xpm_d (widget->window, bm, NULL, xpmarray);
		g_strfreev(xpmarray);
	}
	return pm;
}
*/

void
init_xpms(void)
{
	icon_mainlogo = pixmap_read_from_file (N_("mainlogo.xpm"));
	icon_mainlogo_overnet = pixmap_read_from_file (N_("mainlogo_overnet.xpm"));
	icon_app = pixmap_read_from_file (N_("app.xpm"));
	icon_resume = pixmap_read_from_file (N_("resume.xpm"));
	icon_resumeall = pixmap_read_from_file (N_("resumeall.xpm"));
	icon_pause = pixmap_read_from_file (N_("pause.xpm"));
	icon_pauseall = pixmap_read_from_file (N_("pauseall.xpm"));
	icon_options = pixmap_read_from_file (N_("options.xpm"));
	icon_connect = pixmap_read_from_file (N_("connect.xpm"));
	icon_cancel = pixmap_read_from_file (N_("cancel.xpm"));
	icon_search = pixmap_read_from_file (N_("search.xpm"));
	icon_folder = pixmap_read_from_file (N_("folder.xpm"));	
	icon_status = pixmap_read_from_file (N_("status.xpm"));
	icon_upload = pixmap_read_from_file (N_("upload.xpm"));
	icon_download = pixmap_read_from_file (N_("download.xpm"));
	icon_servers = pixmap_read_from_file (N_("servers.xpm"));
	icon_about = pixmap_read_from_file (N_("about.xpm"));
	icon_stats = pixmap_read_from_file (N_("stats.xpm"));
	icon_threestars = pixmap_read_from_file (N_("threestars.xpm"));
	icon_twostars = pixmap_read_from_file (N_("twostars.xpm"));
	icon_onestar = pixmap_read_from_file (N_("onestar.xpm"));
	icon_extend = pixmap_read_from_file (N_("extend.xpm"));
	icon_200more = pixmap_read_from_file (N_("200more.xpm"));
	dot_yellow = pixmap_read_from_file (N_("dotyellow.xpm"));
	dot_white = pixmap_read_from_file (N_("dotwhite.xpm"));
	dot_red = pixmap_read_from_file (N_("dotred.xpm"));
	dot_purple = pixmap_read_from_file (N_("dotpurple.xpm"));
	dot_green = pixmap_read_from_file (N_("dotgreen.xpm"));
	dot_blue = pixmap_read_from_file (N_("dotblue.xpm"));
	icon_updown = pixmap_read_from_file (N_("updown.xpm"));
	icon_columnhide = pixmap_read_from_file (N_("columnhide.xpm"));
	icon_columnshow = pixmap_read_from_file (N_("columnshow.xpm"));
	icon_skull = pixmap_read_from_file (N_("skull.xpm"));
	icon_core = pixmap_read_from_file(N_("core.xpm"));
	icon_ok = pixmap_read_from_file (N_("ok.xpm"));
	icon_refresh = pixmap_read_from_file (N_("refresh.xpm"));
	icon_wizard = pixmap_read_from_file (N_("wizard.xpm"));
	icon_wizard_off = pixmap_read_from_file(N_("wizard_off.xpm"));
	icon_exit = pixmap_read_from_file (N_("exit.xpm"));
	icon_disconnect = pixmap_read_from_file (N_("disconnect.xpm"));
	icon_faq = pixmap_read_from_file (N_("faq.xpm"));
	icon_mail = pixmap_read_from_file (N_("mail.xpm"));
	icon_ignition = pixmap_read_from_file(N_("ignition.xpm"));
	icon_ignition_small = pixmap_read_from_file(N_("ignition_small.xpm"));
#ifndef LINUX
	g_print (N_("TODO: load specific OS logo instead of Tux here (%s)\n"), __FUNCTION__);
#endif /* ifndef LINUX */
	icon_tux = pixmap_read_from_file (N_("tux.xpm"));
	icon_eyes = pixmap_read_from_file (N_("eyes.xpm"));
	icon_optionon = pixmap_read_from_file (N_("optionon.xpm"));
	icon_optionoff = pixmap_read_from_file (N_("optionoff.xpm"));
	icon_connections = pixmap_read_from_file (N_("connections.xpm"));

	icon_gnome1 = pixmap_read_from_file (N_("gnome1.xpm"));
	icon_gnome2 = pixmap_read_from_file (N_("gnome2.xpm"));
	icon_kde = pixmap_read_from_file (N_("kde.xpm"));

}

void
init_pixmaps (void)
{
	dotyellow_pm = gdk_pixmap_create_from_xpm_d ( clist_downloads->window, &dotyellow_bm, NULL, dot_yellow);
	dotwhite_pm	= gdk_pixmap_create_from_xpm_d ( clist_downloads->window, &dotwhite_bm, NULL, dot_white);
//	dotwhite_pm = pixmap_create_from_xpm_file (clist_downloads, &dotwhite_bm, "icons/default/dotwhite.xpm");
	dotblue_pm	= gdk_pixmap_create_from_xpm_d ( clist_downloads->window, &dotblue_bm, NULL, dot_blue);
	dotgreen_pm	= gdk_pixmap_create_from_xpm_d ( clist_downloads->window, &dotgreen_bm, NULL, dot_green);
	dotpurple_pm = gdk_pixmap_create_from_xpm_d ( clist_downloads->window, &dotpurple_bm, NULL, dot_purple);
	dotred_pm	= gdk_pixmap_create_from_xpm_d ( clist_downloads->window, &dotred_bm, NULL, dot_red);
}


/* icons_get_installed_icon_themes
 *
 * returns a list of installed icon themes (i.e. directories with icons)
 *
 * the first entry returned should automatically be either the selected
 *   icon theme (if option is set) or the default icon theme path (if
 *   option is not set)
 *
 */

GSList *
icons_get_installed_icon_themes (void)
{
	GSList		*themes = NULL;
	gchar		*homedir = g_getenv (N_("HOME"));

	if (option_iconthemedir)
		themes = g_slist_append (themes, g_strdup(option_iconthemedir));

	/* first, check $pkgdatapath/icons/ */

	if (access(APP_PIXMAPS, F_OK)==0)
	{
		DIR	*dir;

		dir = opendir (APP_PIXMAPS);
		if (dir)
		{
			struct dirent *entry;
			while ((entry=readdir(dir)))
			{
				if (entry->d_name[0]!='.')
				{
					gchar *fullpath = g_strdup_printf (N_("%s%c%s"),APP_PIXMAPS, SLASH_CHAR, entry->d_name);
					if ((!option_iconthemedir) || strcmp(option_iconthemedir,fullpath)!=0)
					{
						themes = g_slist_append (themes, fullpath);
//						g_print ("-> '%s'\n", fullpath);
					}
				}
			}
		} else status_system_error_msg (_("opendir(APP_PIXMAPS) failed"));
	} else g_printerr (_("Standard path for icon themes '%s' does not exist!\n"), APP_PIXMAPS);

	if (homedir)
	{
		gchar *hdiconpath = g_strdup_printf (N_("%s%c.%s%cicons"), homedir, SLASH_CHAR, PACKAGE, SLASH_CHAR);
		if (hdiconpath)
		{
			if (access(hdiconpath,F_OK)==0)
			{
				DIR	*dir;

				dir = opendir (hdiconpath);
				if (dir)
				{
					struct dirent *entry;
					while ((entry=readdir(dir)))
					{
						if (entry->d_name[0]!='.')
						{
							gchar *fullpath = g_strdup_printf (N_("%s%c%s"),hdiconpath, SLASH_CHAR, entry->d_name);
							themes = g_slist_append (themes, fullpath);
//							g_print ("-> '%s'\n", fullpath);
						}
					}
				} else status_system_error_msg (_("opendir($HOME/.ed2k_gui/icons/) failed"));
			}
			g_free(hdiconpath);
		}
	}

	return themes;
}


