/* statusbar.c
*
*  functions to display server connection status on statusbar
*
*   (c) 2001-2002 Tim-Philipp Muller <t.i.m@orange.net>
*
*/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// TODO:

#include <string.h>

#include "global.h"
#include "colors.h"
#include "mainwindow.h"
#include "statusbar.h"

static GtkWidget	*statusbar2 = NULL;			/* our statusbar widget         */
static gint			 statusbar_msg_context = 0;	/* statusbars need that somehow */

static void			 statusbar_extra_message_real (const gchar *msg);

/* functions */

static void
statusbar_extra_message_real (const gchar *msg)
{
	g_return_if_fail (msg!=NULL);

	if ((window) && (statusbar2))
	{
		gtk_statusbar_pop (GTK_STATUSBAR(statusbar2), statusbar_msg_context);
		gtk_statusbar_push (GTK_STATUSBAR(statusbar2), statusbar_msg_context, msg);
		/* if we had a two-line statusbar before, we want it to
		 * go back to one line again without switching pages... => resize
		 */
		gtk_container_set_resize_mode(GTK_CONTAINER(statusbar2), GTK_RESIZE_IMMEDIATE);
		gtk_container_check_resize(GTK_CONTAINER(statusbar2));
	}
}


/* statusbar_extra_message
 *
 *
 *
 */

void
statusbar_extra_message (const gchar *format, ...)
{
	GString	*output = NULL;
	gchar	*s_alloc = NULL;
	va_list	 ap;

	g_return_if_fail (format!=NULL);

	output = g_string_new("");
	g_return_if_fail (output!=NULL);

	va_start(ap, format);
	if (vasprintf(&s_alloc, format, ap)>=0)
	{
		output = g_string_append (output, s_alloc);
		free(s_alloc);	/* as vasprintf allocs it, we need to use free() and not g_free() */
		s_alloc = NULL;
	}
	va_end(ap);

	/* make sure there's a space as first character -> looks nicer */
	if (output->str[0] != ' ')
		output = g_string_prepend_c (output, ' ');

	statusbar_extra_message_real (output->str);
	g_string_free(output,TRUE);
}

/* statusbar_create
 *
 *
 */

GtkWidget *
statusbar_create (void)
{
	statusbar2 = gtk_statusbar_new();	// statusbar #2 for messages
	gtk_widget_show (statusbar2);
	statusbar_msg_context = gtk_statusbar_get_context_id (GTK_STATUSBAR(statusbar2), N_("msg"));
	return statusbar2;
}


/* statusbar_set_styles
 *
 *
 */

void
statusbar_set_styles (void)
{
	set_style_recursively (statusbar2, style_dark_green_background);
}



